


jsx3.Class.defineClass("jsx3.app.Cache", null, [jsx3.util.EventDispatcher], function(Cache, Cache_prototype) {

  var Document = jsx3.xml.Document;

  
  Cache.REMOVE = "remove";

  
  Cache.ADD = "add";

  
  Cache.CHANGE = "change";

  
  Cache.ASYNC_TIMEOUT = 60000;

  
  Cache.XSDNS = "http://xsd.tns.tibco.com/gi/cache";

  
  Cache._LOADING_DOC = new Document().loadXML('<loading xmlns="' + Cache.XSDNS + '"/>');
  
  Cache._TIMEOUT_DOC = new Document().loadXML('<timeout xmlns="' + Cache.XSDNS + '"/>');
  
  Cache._ERROR_DOC = new Document().loadXML('<error xmlns="' + Cache.XSDNS + '"/>');

  
  Cache_prototype.init = function() {
    // declare an object array to hold all cache documents
    
    this._index = {};
    
    this._parents = [];
  };
  
  Cache_prototype.addParent = function(objParent) {
    this._parents.push(objParent);
  };

  
  Cache_prototype.clearById = function(strId) {
    var objRecord = this._index[strId];
    if (objRecord) {
      delete this._index[strId];
      this.publish({subject:strId, action:Cache.REMOVE});
      this.publish({subject:Cache.CHANGE, id:strId, action:Cache.REMOVE});
      return objRecord.jsxdocument;
    }
  };



  
  Cache_prototype.isSystem = function(strId) {
    return false;
//    return this._index[strId] != null && this._index[strId].jsxsystem;
  };



  
  Cache_prototype.clearByTimestamp = function(intTimestamp) {
    if (intTimestamp instanceof Date) intTimestamp = intTimestamp.getTime();

    var changed = false;
    var ids = [];

    for (var p in this._index) {
      var record = this._index[p];
      if (record.jsxtimestamp < intTimestamp) {
        delete this._index[p];
        this.publish({subject:p, action:Cache.REMOVE});
        ids.push(p);
      }
    }

    if (ids.length > 0)
      this.publish({subject:Cache.CHANGE, ids:ids, action:Cache.REMOVE});

    return ids;
  };

  
  Cache_prototype.getDocument = function(strId) {
    if (this._index[strId] != null)
      return this._index[strId].jsxdocument;
    
    for (var i = 0; i < this._parents.length; i++) {
      var doc = this._parents[i].getDocument(strId);
      if (doc != null) return doc;
    }

    return null;
  };
  
  
  Cache_prototype.getOrOpenDocument = function(strURL, strId, objClass) {
    if (strId == null) strId = strURL.toString();
    return this.getDocument(strId) || this._openDocument(strURL, strId, objClass, false);
  };

  
  Cache_prototype.openDocument = function(strURL, strId, objClass) {
    return this._openDocument(strURL, strId, objClass, false);
  };

  
  Cache_prototype.getOrOpenAsync = function(strURL, strId, objClass) {
    if (strId == null) strId = strURL.toString();
    return this.getDocument(strId) || this._openDocument(strURL, strId, objClass, true);
  };

  
  Cache_prototype._openDocument = function(strURL, strId, objClass, bAsync) {
    if (objClass == null) objClass = Document.jsxclass;
    if (strId == null) strId = strURL.toString();

    var objXML = objClass.newInstance();
    objXML.setAsync(bAsync);

    if (bAsync) {
      objXML.subscribe("*", this, "_onAsyncDone");
      objXML._jsxcacheid = strId;
      objXML.load(strURL, Cache.ASYNC_TIMEOUT);
      objXML = Cache._LOADING_DOC.cloneDocument();
    } else {
      objXML.load(strURL);
    }

    this.setDocument(strId, objXML);
    return objXML;
  };

  
  Cache_prototype._onAsyncDone = function(objEvent) {
    var objXML = objEvent.target;
    var strEvtType = objEvent.subject;
    var strId = objXML._jsxcacheid;

    delete objXML._jsxcacheid;
    objXML.unsubscribe("*", this);

    if (this._index) {
      if (strEvtType == Document.ON_RESPONSE) {
        this.setDocument(strId, objXML);
      } else if (strEvtType == Document.ON_TIMEOUT) {
        this.setDocument(strId, Cache._TIMEOUT_DOC.cloneDocument());
      } else if (strEvtType == Document.ON_ERROR) {
        var d = Cache._ERROR_DOC.cloneDocument();
        d.setAttribute("error", objXML.getError().toString());
        this.setDocument(strId, d);
      }
    }
  };

  
  Cache_prototype.setDocument = function(strId, objDocument) {
    if (strId == null) throw new jsx3.IllegalArgumentException("strId", strId);
    if (objDocument == null) throw new jsx3.IllegalArgumentException("objDocument", objDocument);
    
    //create a new cache object
    var record = {};
    
    record.jsxtimestamp = (new Date()).getTime();
    
    record.jsxdocument = objDocument;

    // check whether a document already exists in the cache
    var evtAction = this._index[strId] ? Cache.CHANGE : Cache.ADD;

    //persist to cache
    this._index[strId] = record;

    this.publish({subject:strId, action:evtAction});
    this.publish({subject:Cache.CHANGE, action:evtAction, id:strId});
  };

  
  Cache_prototype.getTimestamp = function(strId) {
    var record = this._index[strId];
    return record != null ? record.jsxtimestamp : null;
  };

  
  Cache_prototype.keys = function() {
    var keys = [];
    for (var f in this._index)
      keys[keys.length] = f;
    return keys;
  };
  
  
  Cache_prototype.destroy = function() {
    delete this._index;
    delete this._parents;
  };
  
});




jsx3.Cache = jsx3.app.Cache;


