


jsx3.Class.defineClass('jsx3.app.Monitor', jsx3.util.Logger.FormatHandler, null, function(Monitor, Monitor_prototype) {

  var Server = jsx3.app.Server;
  
  
  Monitor._IDE_LOADED = false;
  
  Monitor._URL = jsx3.net.URIResolver.DEFAULT.resolveURI("jsx:///html/jsx3.app.Monitor.html");
  
  
  Monitor.ideDidLoad = function() {
    Monitor._IDE_LOADED = true;
  };
          
  
  Monitor_prototype._on = false;
  
  Monitor_prototype._disableInIDE = true;
  
  Monitor_prototype._serverNamespace = null;
  
  Monitor_prototype._activateOnHotKey = false;
  
  Monitor_prototype._server = null;
  
  Monitor_prototype._window = null;

  
  Monitor_prototype.init = function(strName) {
    this.jsxsuper(strName);
  };

  Monitor_prototype.onAfterInit = function() {
    if (this._serverNamespace != null) {
      var app = null;
      if (jsx3.Class.forName("jsx3.lang.System"))
        app = jsx3.System.getApp(this._serverNamespace);

      if (app != null) {
        this._initServer(app);
      } else {
        // register for when servers are created so we know if "our" server is created
        Server.subscribe(Server.INITED, this, "serverDidLoad");
      }
    } else {
      this._disableInIDE = false;
      this._activateOnHotKey = false;
      this._on = true;
      this._openWindow();
    }
  };
  
  
  Monitor_prototype._initServer = function(objServer) {
    this._on = true;
    this._server = objServer;
    
    if (this._activateOnHotKey) {
      // register for ctrl+alt+m if we only activate on hot key
      var me = this;
      objServer.registerHotKey(function(objEvent){ me.onHotKey(); }, "m", false, true, true);
    } else {
      this._openWindow();
    }
  };
  
  
  Monitor_prototype.serverDidLoad = function(objEvent) {
    var objServer = objEvent.target;

    if (objServer.getEnv("namespace") == this._serverNamespace) {
      if (!Monitor._IDE_LOADED || !this._disableInIDE)
        this._initServer(objServer);
      
      // unsubscribe: only one server per monitor is allowed
      Server.unsubscribe(Server.INITED, this);
    }
  };

  
  Monitor_prototype.handle = function(objRecord) {
    if (this._on && (!Monitor._IDE_LOADED || !this._disableInIDE)) {
      var w = this._window;

      if (w) {
        if (w.closed) {
          if (!this._activateOnHotKey)
            this._openWindow();
        }

        if (!w.closed && w.appendMessage) {
          // When first opening the app monitor, push all in-memory records to the output...
          if (w.isFirstTime()) {
            var manager = jsx3.util.Logger.Manager.getManager();
            var memoryHandler = manager.getHandler("memory");
            if (memoryHandler) {
              jsx3.$A(memoryHandler.getRecords()).each(jsx3.$F(function(e) {
                if (e !== objRecord)
                  w.appendMessage(this.format(e), jsx3.util.Logger.levelAsString(e.getLevel()));
              }).bind(this));
            }
          }

          w.appendMessage(this.format(objRecord), jsx3.util.Logger.levelAsString(objRecord.getLevel()));
        }
      }
    }
  };
  
  
  Monitor_prototype.onHotKey = function() {
    if (this._window == null || this._window.closed)
      this._openWindow();
  };
  
  
  Monitor_prototype._openWindow = function() {
    this._window = window.open(Monitor._URL, "Monitor_" + this.getName(),
          "directories=no," + 
          "location=no," + 
          "menubar=no," + 
          "status=yes," + 
          "personalbar=no," + 
          "titlebar=yes," + 
          "toolbar=no," +
          "resizable=yes," +
          "scrollbars=no," + 
          "width=500," + 
          "height=400");

    if (this._window) {
      if (this._server) {
        if (typeof(this._window.setName) == "function")
          this._window.setName(this._server.getEnv('namespace'));
        else
          this._window._jsxname = this._server.getEnv('namespace');
      }

      window.focus();
    }
  };
  
  
  Monitor_prototype.getDisableInIDE = function() {
    return this._disableInIDE;
  };
  
  
  Monitor_prototype.setDisableInIDE = function(disableInIDE) {
    this._disableInIDE = disableInIDE;
  };
  
  
  Monitor_prototype.getServerNamespace = function() {
    return this._serverNamespace;
  };
  
  
  Monitor_prototype.setServerNamespace = function(serverNamespace) {
    this._serverNamespace = serverNamespace;
  };
  
  
  Monitor_prototype.getActivateOnHotKey = function() {
    return this._activateOnHotKey;
  };
  
  
  Monitor_prototype.setActivateOnHotKey = function(activateOnHotKey) {
    this._activateOnHotKey = activateOnHotKey;
  };
  
});

jsx3.util.Logger.Handler.registerHandlerClass(jsx3.app.Monitor.jsxclass);
