


jsx3.Class.defineClass("jsx3.app.Properties", null, null, function(Properties, Properties_prototype) {

  var LOG = jsx3.util.Logger.getLogger(Properties.jsxclass.getName());

  
  Properties.GLOBAL_KEY = "_global";

  
  Properties_prototype.init = function() {
    
    this._parents = [];
    
    this._children = [];
    
    this._order = [Properties.GLOBAL_KEY];
    
    this._docs = {};
    this._docs[Properties.GLOBAL_KEY] = {};
    
    this._master = {};
    
    this._masterDirty = false;
    
    this._parentsMaster = {};
    
    this._parentsDirty = false;
  };

  
  Properties_prototype.loadXML = function(objXML, strId) {

    var t1 = new jsx3.util.Timer(Properties.jsxclass, strId ||
        (objXML instanceof jsx3.xml.Document ? objXML.getSourceURL() : objXML.getNodeName()));


    if (strId == null) {
      strId = Properties.GLOBAL_KEY;
    } else if (jsx3.util.arrIndexOf(this._order, strId) < 0) {
      this._order.splice(1, 0, strId);
    }

    var doc = this._docs[strId];
    if (doc == null)
      doc = this._docs[strId] = {};

    for (var i = objXML.selectNodeIterator("./record"); i.hasNext(); ) {
      var next = i.next();

      var id = next.getAttribute("jsxid");
      var eval = next.getAttribute("eval");
      var text = next.getAttribute("jsxtext");

      if (eval == "1" || eval == "true") {
        try {
          text = isNaN(text) ? jsx3.eval(text) : Number(text);
        } catch (e) {
          LOG.warn(jsx3._msg("props.eval", id, text), jsx3.NativeError.wrap(e));
        }
      }
      doc[id] = text;
    }

    // The cache and the parent cache of each child are dirty
    this._masterDirty = true;


    t1.log("load");

  };

  
  Properties_prototype.addParent = function(objParent) {
    this._parents.splice(0, 0, objParent);
    objParent._children.push(this);

    // Parent cache is dirty
    this._setChildrenDirty(true);
  };

  
  Properties_prototype.removeParent = function(objParent) {
    var index = jsx3.util.arrIndexOf(this._parents, objParent);
    if (index >= 0) {
      this._parents.splice(index, 1);
      objParent._removeChild(this);

      // Parent cache is dirty
      this._setChildrenDirty(true);
    }
  };

  
  Properties_prototype._removeChild = function(objChild) {
    var index = jsx3.util.arrIndexOf(this._children, objChild);
    if (index >= 0)
      this._children.splice(index, 1);
  };

  
  Properties_prototype.removeAllParents = function() {
    if (this._parents.length > 0) {
      for (var i = 0; i < this._parents.length; i++)
        this._parents[i]._removeChild(this);
      this._parents = [];

      this._parentsDirty = false;
      this._parentsMaster = {};
    }
  };

  
  Properties_prototype.getParents = function() {
    return this._parents.concat();
  };

  
  Properties_prototype.containsKey = function(strKey) {
    if (this._masterDirty)
      this._reconstructMaster();

    return typeof(this._master[strKey]) != "undefined";
  };

  
  Properties_prototype.getKeys = function() {
    if (this._masterDirty)
      this._reconstructMaster();

    var keys = [];
    for (var f in this._master) keys[keys.length] = f;
    return keys;
  };

  
  Properties_prototype.get = function(strKey) {
    if (this._masterDirty)
      this._reconstructMaster();

    if (typeof(this._master[strKey]) != "undefined")
      return this._master[strKey];

    if (this._parentsDirty)
      this._reconstructParentsMaster();

    return this._parentsMaster[strKey];
  };

  
  Properties_prototype.set = function(strKey, strValue) {
    if (typeof(strValue) == "undefined") throw new jsx3.IllegalArgumentException("strValue", strValue);
    // 1. set the value in the memory space
    this._docs[Properties.GLOBAL_KEY][strKey] = strValue;

    // 2. update the cache
    this._master[strKey] = strValue;

    // 3. tell children that a parent cache is dirty
    this._setChildrenDirty();
  };

  
  Properties_prototype.remove = function(strKey) {
    // 1. remove the value from all memory spaces
    for (var key in this._docs)
      delete this._docs[key][strKey];

    // 2. update the cache
    delete this._master[strKey];

    // 3. tell children that a parent cache is dirty
    this._setChildrenDirty();
  };

  
  Properties_prototype._reconstructMaster = function() {
    this._masterDirty = false;
    var master = this._master = {};

    for (var i = this._order.length - 1; i >= 0; i--) {
      var doc = this._docs[this._order[i]];
      for (var f in doc)
        master[f] = doc[f];
    }
  };

  
  Properties_prototype._reconstructParentsMaster = function() {
    this._parentsDirty = false;
    var master = this._parentsMaster = {};

    for (var i = this._parents.length - 1; i >= 0; i--) {
      var p = this._parents[i];
      if (p._masterDirty)
        p._reconstructMaster();
      if (p._parentsDirty)
        p._reconstructParentsMaster();

      var m1 = p._master;
      var m2 = p._parentsMaster;

      for (var f in m2)
        master[f] = m2[f];
      for (var f in m1)
        master[f] = m1[f];
    }
  };

  
  Properties_prototype._setChildrenDirty = function(bMe) {
    var stack = bMe ? [this] : this._children.concat();

    while (stack.length > 0) {
      var p = stack.shift();
      if (!p._parentsDirty) {
        p._parentsDirty = true;
        stack.push.apply(stack, p._children);
      }
    }
  };

});
