

// @jsxobf-clobber  _locale _path


jsx3.Class.defineClass("jsx3.app.PropsBundle", jsx3.app.Properties, null, function(PropsBundle, PropsBundle_prototype) {

  var LOG = jsx3.util.Logger.getLogger(PropsBundle.jsxclass.getName());
  var Job = jsx3.util.Job;

  
  PropsBundle.PATH_SEP = ".";

  
  PropsBundle.ERROR = -1;
  
  
  PropsBundle._PATH_TO_LOCALES = {};

  
  PropsBundle._KEY_TO_PROPS = {};

  
  PropsBundle._GRAPH = new jsx3.util.JobGraph();

  
  PropsBundle._EMPTY = new PropsBundle();
  
  
  PropsBundle.getProps = function(strBasePath, objLocale, objCache) {
    if (!objLocale) objLocale = jsx3.System.getLocale();
    var key = strBasePath + "::" + objLocale;
    
    if (!PropsBundle._KEY_TO_PROPS[key]) {
      var graph = PropsBundle._GRAPH;
      
      if (graph.node(strBasePath))
        return PropsBundle._EMPTY;
      
      var job = new Job(strBasePath);
      PropsBundle._getProps(strBasePath, objLocale, objCache, false, job);
    }
    
    var p = PropsBundle._KEY_TO_PROPS[key];
    if (p == PropsBundle.ERROR)
      throw new jsx3.Exception(jsx3._msg("propbn.err_key", strBasePath, objLocale));
      
    return p;
  };
  
  
  PropsBundle.getPropsAsync = function(strBasePath, objLocale, fctCallback, objCache) {
    if (!objLocale) objLocale = jsx3.System.getLocale();
    var key = strBasePath + "::" + objLocale;
    var job = new Job(null, function() {
      var p = PropsBundle._KEY_TO_PROPS[key];
      fctCallback(p != PropsBundle.ERROR ? p : null);
    });
    PropsBundle._getProps(strBasePath, objLocale, objCache, true, job);
  };

  
  PropsBundle._getProps = function(strBasePath, objLocale, objCache, bAsync, objDoneJob) {
    var graph = PropsBundle._GRAPH;
    strBasePath = strBasePath.toString();

    graph.pause();
    
    graph.add(objDoneJob); // NOTE: graph must be paused
    
    var nextJob = objDoneJob;
    var searchPath = objLocale.getSearchPath();
    
    // For each locale to load, en_US, en, root...
    for (var i = 0; i < searchPath.length; i++) {
      var thisLocale = searchPath[i];
      var thisKey = strBasePath + "::" + thisLocale;
      
      // Check if the locale is already loaded, if so, we're done
      if (PropsBundle._KEY_TO_PROPS[thisKey]) 
        break;

      // Check if the locale is already queued
      var thisJob = graph.node(thisKey);
      
      if (! thisJob) {
        // If not, queue loading the locale
        thisJob = PropsBundle._makeJob(thisKey, strBasePath, thisLocale, objCache, bAsync);
        graph.add(thisJob);
        thisJob.add(nextJob);
      } else {
        // Otherwise, just create the dependency, and we're done
        thisJob.add(nextJob);
        break;
      }      

      nextJob = thisJob;
    }
    
    graph.start();
  };
  
  
  PropsBundle._makeJob = function(thisKey, strBasePath, thisLocale, objCache, bAsync) {
    return new Job(thisKey, function() {
      var me = this;
      PropsBundle._loadBundle(thisKey, strBasePath, thisLocale, objCache, bAsync, function() {
        me.finish();
      });
      return Job.WAIT; // Ok for sync too since finish() is called before this returns WAIT.
    });    
  };
  
  
  PropsBundle._loadBundle = function(strKey, strBasePath, objLocale, objCache, bAsync, fctCallback) {
    var propsCache = PropsBundle._KEY_TO_PROPS, localeCache = PropsBundle._PATH_TO_LOCALES;
    var bRoot = false, bDone = false;
    
    if (propsCache[strKey]) {
      bDone = true;
    } else {
      if (objLocale.toString() == "") {
        bRoot = true;
      } else {
        if (! localeCache[strBasePath][objLocale]) {
          var searchPath = objLocale.getSearchPath();
          for (var i = 1; !propsCache[strKey] && i < searchPath.length; i++) {
            var ancestorLocale = searchPath[i];
            var ancestorKey = strBasePath + "::" + ancestorLocale;
            propsCache[strKey] = propsCache[ancestorKey];
          }
          bDone = true;
        }
      }
    }
    
    if (bDone) {
      // QUESTION: timeout for async?
      fctCallback();
      return;
    }
    
    var strUrl;
    if (bRoot) {
      localeCache[strBasePath] = {};
      strUrl = strBasePath;
    } else {
      var index = strBasePath.lastIndexOf(".");
      strUrl = strBasePath.substring(0, index) + PropsBundle.PATH_SEP + objLocale + strBasePath.substring(index);
    }

    var doc = null, bAlreadyLoaded = false;
    
    if (objCache)
      doc = objCache.getDocument(strUrl);

    if (doc) {
      bAlreadyLoaded = true;      
    } else {
      doc = new jsx3.xml.Document();
      
      if (bAsync) {
        doc.setAsync(true);
        doc.subscribe("*", PropsBundle, function(objEvent) {
          PropsBundle._loadBundle2(strKey, strBasePath, objLocale, objCache, objEvent.target, fctCallback);
        });
      } else {
        bAlreadyLoaded = true;
      }

      doc.load(strUrl);
    }
    
    if (bAlreadyLoaded)
      PropsBundle._loadBundle2(strKey, strBasePath, objLocale, objCache, doc, fctCallback);
  };
  
  
  PropsBundle._loadBundle2 = function(strKey, strBasePath, objLocale, objCache, objDoc, fctCallback) {
    var strUrl = objDoc.getSourceURL();
    if (! objDoc.hasError()) {
      if (objCache)
        objCache.setDocument(strUrl, objDoc);
      PropsBundle._addLocaleRefs(strBasePath, objDoc);
      PropsBundle._loadInlineLocales(strBasePath, objLocale, objDoc);
    } else {
      LOG.error(jsx3._msg("propbn.err_file", strUrl, objDoc.getError()));
      PropsBundle._KEY_TO_PROPS[strKey] = PropsBundle.ERROR;
    }
    
    fctCallback();
  };

  
  PropsBundle._loadInlineLocales = function(strBasePath, objLocale, objXML) {
    for (var i = objXML.selectNodeIterator("/data/locale"); i.hasNext(); ) {
      var lNode = i.next();
      var localeKey = lNode.getAttribute("key") || "";

      PropsBundle._setLocaleDoc(strBasePath, localeKey, lNode);
      PropsBundle._PATH_TO_LOCALES[strBasePath][localeKey] = true;
    }

    // Default to loading the entire external file for the requested locale.
    if (! PropsBundle._KEY_TO_PROPS[strBasePath + "::" + objLocale]) {
      PropsBundle._setLocaleDoc(strBasePath, objLocale.toString(), objXML);
    }
  };

  
  PropsBundle._setLocaleDoc = function(strBasePath, strLocale, objXML) {
    var propsCache = PropsBundle._KEY_TO_PROPS;
    
    var props = new PropsBundle();
    props.loadXML(objXML);
    props._path = strBasePath;
    props._locale = jsx3.util.Locale.valueOf(strLocale);

    propsCache[strBasePath + "::" + strLocale] = props;
    
    // If not the root locale, then add a reference to the parent locale, according to the locale search path.
    // This requires the the parent locale always be loaded before the descendant locale.
    if (strLocale) {
      var path = props._locale.getSearchPath();
      for (var i = 1; i < path.length; i++) {
        var parent = propsCache[strBasePath + "::" + path[i]];
        if (parent) {
          props.addParent(parent);
          return;
        }
      }
      
      LOG.error("Parent of bundle " + strBasePath + " (" + strLocale + ") is null.");
    }
  };

  
  PropsBundle._addLocaleRefs = function(strBasePath, objXML) {
    var localeString = objXML.getAttribute("locales");

    if (PropsBundle._PATH_TO_LOCALES[strBasePath] == null)
      PropsBundle._PATH_TO_LOCALES[strBasePath] = {};

    if (localeString != null) {
      var localeKeys = localeString.split(/\s*,\s*/);
      for (var i = 0; i < localeKeys.length; i++)
        if (localeKeys[i])
          PropsBundle._PATH_TO_LOCALES[strBasePath][localeKeys[i]] = true;
    }
  };

  
  PropsBundle_prototype.getLocale = function() {
    return this._locale;
  };

  
  PropsBundle_prototype.getPath = function() {
    return this._path;
  };

  
  PropsBundle.clearCache = function(strPath, objCache) {
    if (strPath) {
      delete PropsBundle._PATH_TO_LOCALES[strPath];

      var prefix = strPath + "::";
      for (var f in PropsBundle._KEY_TO_PROPS) {
        if (f.indexOf(prefix) == 0)
          delete PropsBundle._KEY_TO_PROPS[f];
      }

      if (objCache) {
        var cacheKeys = objCache.keys();
        for (var i = 0; i < cacheKeys.length; i++) {
          if (cacheKeys[i].indexOf(strPath) == 0)
            objCache.clearById(cacheKeys[i]);
        }
      }
    } else {
      PropsBundle._PATH_TO_LOCALES = {};
      PropsBundle._KEY_TO_PROPS = {};
    }
  };

});
