

// @jsxobf-clobber-shared  _jsxns _jsxroots


jsx3.Class.defineClass("jsx3.app.Server", null, [jsx3.util.EventDispatcher, jsx3.net.URIResolver],
    function(Server, Server_prototype) {

  jsx3.util.EventDispatcher.jsxclass.mixin(Server);

  var Event = jsx3.gui.Event;
  var URIResolver = jsx3.net.URIResolver;
  var Browser = jsx3.app.Browser;
  var Block = null;

  var LOG = jsx3.util.Logger.getLogger(Server.jsxclass.getName());

  
  Server.Z_DIALOG = 2;
  
  Server.Z_DRAG = 3;
  
  Server.Z_MENU = 4;

  
  Server.INITED = "inited";

  
  Server.HELP = "help";

  
  Server._DIVID = "JSX";

  
  Server._INSTANCES = new jsx3.util.List();

  
  Server_prototype.JSXROOT = null;

  
  Server_prototype.JSXBODY = null;

  
  Server_prototype.Cache = null;

  
  Server_prototype.ENVIRONMENT = null;

  
  Server_prototype.DOM = null;

  
  Server_prototype.JSS = null;

  
  Server_prototype.init = function(strAppPath, objGUI, bPaint, objEnv) {
    //instance a new DOM controller for this server instance that will track and manage JSX GUI and Data objects
    this.DOM = new jsx3.app.DOM();

    //instance the cache controller -- although this object is little-changed, its recommended use is much more limited and can merely be thought of as a simplified way to cache an XML document or system XSLT
    this.Cache = new jsx3.app.Cache();
    this.Cache.addParent(jsx3.getSharedCache());

    if (objEnv != null && objEnv.jsxsettings != null) {
      this._jsxsettings = objEnv.jsxsettings;
      delete objEnv.jsxsettings;
    }

    this.ENVIRONMENT = objEnv = objEnv != null ? jsx3.clone(objEnv) : {};
    objEnv.apppath = strAppPath.replace(/\/*$/, "");

    //load the application config file, so we know which resource files (JS, CSS, XML, etc) to load
    var objSettings = this.getSettings();

    var settingsTree = objSettings.get();

    for (var f in settingsTree) {
      var key = f.toLowerCase();
      if (typeof(objEnv[key]) == "undefined" && typeof(settingsTree[f]) != "object") {
        objEnv[key] = settingsTree[f];
      }
    }

    // figure out path to resources, support for environment variable jsxappbase which is a relative (or absolute)
    // path from the config.xml file to the content root of the application
    objEnv.apppathuri = new jsx3.net.URI(objEnv.apppath + "/");
    if (objEnv.jsxappbase)
      objEnv.apppathuri = objEnv.apppathuri.resolve(objEnv.jsxappbase);

    var i = objEnv.apppath.indexOf(jsx3.APP_DIR_NAME);
    if (i >= 0)
      objEnv.apppathrel = objEnv.apppath.substring(i + jsx3.APP_DIR_NAME.length + 1);
    objEnv.apppathabs = Browser.getLocation().resolve(objEnv.apppathuri);

    if (objEnv['liquid'] == null) objEnv['liquid'] = true;

    if (objEnv['eventsvers'] == null) objEnv['eventsvers'] = 3.0;

    if (objEnv['jsxversion'] == null) objEnv['jsxversion'] = "3.1";

    objEnv.abspath = jsx3.getEnv("jsxabspath");
    objEnv.guiref = objGUI;            //object ref to on-screen containing object (div, td, span, etc)
    objEnv.namespace = objEnv.jsxappns || objEnv.namespace;

    if (objEnv.namespace == null)
      throw new jsx3.Exception(jsx3._msg("serv.no_ns", strAppPath));
    
    if (objGUI && this.getEnv("caption"))
      objGUI.ownerDocument.title = this.getEnv("caption");

    // implicitly cancel (trap) all right-click events unless explictly told not to
    if (objGUI && this.getEnv("cancelrightclick"))
      objGUI.ownerDocument.oncontextmenu = new Function("return false;");

    // implicitly cancel (trap) all browser-native errors unless explictly told not to
    if (this.getEnv("cancelerror"))
      jsx3.NativeError.initErrorCapture();

    // add this to the list of all instances
    Server._INSTANCES.add(this);

    //bind a reference to this jsxserver instance to the top-level window, using the declared namespace for this server; this provides a global handle to the server by name and is critical to backwards compatibility
    jsx3.registerApp(this);

    //create JSS MODEL CONTAINER (the dynamic properties object array)
    this.JSS = new jsx3.app.Properties();
    this.LJSS = new jsx3.app.Properties();
    this.JSS.addParent(this.LJSS); // add localized properties to the tree
    this.JSS.addParent(jsx3.System.JSS); // copy system JSS properties into this server

    Server.publish({subject: Server.INITED, target:this});

    //7) if specified, immediately render the view (typically only happens if the user chooses to initialize a new server AFTER the document BODY has fully loaded
    if (bPaint) this.paint();
  };

  
  Server_prototype._createRootAndBody = function() {
    jsx3.require("jsx3.gui.Block");
    Block = jsx3.gui.Block;

    if (this.JSXROOT) return;

    // always create root and body; even though the app might not have a VIEW, it still may want access to the MODEL --
    // even for GUI objects
    var root = this.JSXROOT = this._createRoot("JSXROOT");
    root.setDynamicProperty("jsxbgcolor","@Solid DarkShadow");
    root.setRelativePosition(Block.ABSOLUTE);
    root.setOverflow(Block.OVERFLOWEXPAND);
    root.setAlwaysCheckHotKeys(true);
    root.setIndex(1);

    var body = this.JSXBODY = new Block("JSXBODY",0,0,"100%","100%","");
    body.setDynamicProperty("jsxbgcolor","@Solid Light");
    body.setRelativePosition(Block.ABSOLUTE);
    body.setZIndex(1);
    body.setOverflow(Block.OVERFLOWEXPAND).setIndex(0);
    root.setChild(body);
  };

  
  Server_prototype.onResize = function() {
 if (jsx3.CLASS_LOADER.IE) {
    //use delay in IE to stop CPU thrashing
    window.clearTimeout(this.resize_timeout_id);
    var my = this;
    this.resize_timeout_id = window.setTimeout(function() { my._onResizeDelay(); }, 250);
 } else {
    this._onResizeDelay();
 }
  };

  
  Server_prototype._onResizeDelay = function() {
    //jsx3.util.Logger.getLogger("jsx3.boxmodel").trace("RESIZE " + this.getClass() + ": [" + this.getEnv("NAMESPACE") + "]");

    //get the HTML element that contains this server instance
    var objGUI = this.getEnv("GUIREF");

    //tell the root block to update its profile
    if(objGUI) {
      this.getRootBlock().syncBoxProfile({left:0,top:0,parentwidth:objGUI.clientWidth,parentheight:objGUI.clientHeight}, true);
    }
  };

  
  Server_prototype.getNextZIndex = function(intType) {
    if (this.ZRANGE == null) {
      
      this.ZRANGE = [];
      this.ZRANGE[0] = 0;
      this.ZRANGE[1] = 1000;
      this.ZRANGE[Server.Z_DIALOG] = 5000;
      this.ZRANGE[Server.Z_DRAG] = 7500;
      this.ZRANGE[Server.Z_MENU] = 10000;
      this.ZRANGE[5] = 25000;
    }

    return this.ZRANGE[intType]++;
  };

  
  Server.allServers = function() {
    return Server._INSTANCES.toArray();
  };

  
  Server_prototype._createRoot = function(strName) {
    if (this._jsxroots == null) this._jsxroots = [];

    var root = new Block(strName, 0, 0, "100%", "100%", "");
    root._jsxns = this.getEnv("NAMESPACE");
    root._jsxid = jsx3.app.DOM.newId(this.getEnv("NAMESPACE"));
    root._jsxserver = this;
    this.DOM.add(root);

    this._jsxroots.push(root);
    return root;
  };

  
  Server_prototype.getInvisibleRoot = function() {
    if (this.INVISIBLE == null)
      this.INVISIBLE = this._createRoot("JSXINVISIBLE");
    return this.INVISIBLE;
  };

  
  Server_prototype.getEnv = function(strEnvKey) {
    return this.ENVIRONMENT[strEnvKey.toLowerCase()];
  };

  
  Server._showEnv = function(objEvent) {
    //get the relevant variables to profile
    var s = ['<div class="jsx30block jsx30env">'];
    s.push('<b>Version:</b> ', '@gi.version.full@', '<br/>');
    s.push('<b>XML Version:</b> ', jsx3.getXmlVersion(), '<br/>');
    s.push('<b>System Locale:</b> ', jsx3.System.getLocale().getDisplayName(), '<br/>');
    s.push('<b>Browser:</b> ', jsx3.CLASS_LOADER + '<br/>');
    s.push('<b>Operating System:</b> ', jsx3.app.Browser.getSystem() + " (" + navigator.userAgent + ")");
    s.push('<hr/>');

    var root = null;

    var servers = Server.allServers();
    for (var i = 0; i < servers.length; i++) {
      var server = servers[i];
      s.push("<b>", server.getEnv("namespace"), "</b>", "<div>");
      s.push('<b>Path:</b> ', server.getAppPath(), '<br/>');
      s.push('<b>Version:</b> ', server.getEnv("version"), '<br/>');
      s.push("</div>");

      if (root == null) {
        root = server.getRootBlock();
        if (root.getRendered() == null) root = null;
      }
    }

    s.push('</div>');
    s = s.join("");

    if (root)
      root.showSpy(s, Math.round(root.getRendered().clientWidth/2), Math.round(root.getRendered().clientHeight/2-100));
    else window.alert(s);
  };

  
  Server_prototype.getSettings = function() {
    if (this._jsxsettings == null) {
      var objXML = this.getCache().getOrOpenDocument(jsx3.resolveURI(this.getAppPath() + "/" + jsx3.CONFIG_FILE), "JSX_SETTINGS");
      if (objXML.hasError()) {
        LOG.error(jsx3._msg("serv.err_set", this, objXML.getError()));
        objXML = null;
      }
      
      this._jsxsettings = new jsx3.app.Settings(objXML);
    }
    return this._jsxsettings;
  };

  Server_prototype.getAppPath = function() {
    return this.getEnv("apppath");
  };

  
  Server._taskBarFinder = function(obj) {
    return (obj instanceof jsx3.gui.WindowBar) && obj.getType() == jsx3.gui.WindowBar.TYPETASK;
  };

  
  Server_prototype.getTaskBar = function(objJSX) {
    if (! jsx3.gui.WindowBar) return null;
    if (objJSX == null) objJSX = this.JSXROOT;
    return objJSX.findDescendants(Server._taskBarFinder, false, false, false, true);
  };

  
  Server_prototype.destroy = function() {
    var unloadScript = this.getEnv("onunload");
    if (unloadScript) {
      try {
        this.eval(unloadScript);
      } catch (e) {
        LOG.error(jsx3._msg("serv.err_onun", this), jsx3.NativeError.wrap(e));
      }
    }

    //unsubscribe self from the box profiler
    if (jsx3.Class.forName("jsx3.gui.Painted"))
      jsx3.gui.Painted.Box.unregisterServer(this,this.getEnv("LIQUID"));

    //destroy root view (JSXROOT)
    if (this.JSXROOT) {
      var objGUI = this.JSXROOT.getRendered();
      if (objGUI) {
        //depending upon the server mode (ide or runtime), the outer container may or may not be present
        if (objGUI.parentNode.id == Server._DIVID)
          objGUI = objGUI.parentNode;
        jsx3.html.removeNode(objGUI);
      }
    }

    //free-up GUI objects belonging to the various roots
    if (this._jsxroots) {
      for (var i = 0; i < this._jsxroots.length; i++) {
        var root = this._jsxroots[i];
        root.removeChildren();
      }
    }
    delete this._jsxroots;

    //shut down subscriptions to dom and cache
    this.DOM.unsubscribeAllFromAll();
    this.DOM.destroy();

    //free-up xml resources
    this.Cache.unsubscribeAll(jsx3.app.Cache.CHANGE);
    this.Cache.destroy();

    //remove ref to this server
    jsx3.deregisterApp(this);

    // remove instance from list of all Server instances
    Server._INSTANCES.remove(this);

    // uncapture BODY keydown
    Event.unsubscribe(Event.KEYDOWN, this, "checkHotKeys");

    this.ENVIRONMENT = {};
  };

  
  Server_prototype.paint = function(objXML) {
    jsx3.require("jsx3.gui.Alerts", "jsx3.gui.Block");
    Block = jsx3.gui.Block;

    // make sure that Server implements jsx3.gui.Alerts before painting
    if (! jsx3.gui.Alerts.jsxclass.isAssignableFrom(Server.jsxclass))
      Server.jsxclass.addInterface(jsx3.gui.Alerts.jsxclass);

    //3) register the browser mode
    jsx3.html.getMode(this.getEnv("GUIREF"));

    //4) generate the initial DOM structure (JSXROOT AND JSXBODY)
    this._createRootAndBody();

    //5) register the server with the box manager; pass the liquid mode (true or false)
    jsx3.gui.Painted.Box.registerServer(this, this.getEnv("LIQUID"));

    if (this.getEnv("BODYHOTKEYS"))
      Event.subscribe(Event.KEYDOWN, this, "checkHotKeys");

    // Register ctrl+alt+shift+j to show the server environment
    var showEnv = "_showEnv";
    this.registerHotKey(new Function("jsx3.app.Server." + showEnv + "();"), 74, true, true, true);
    // Register the esc key to abort DnD
    this.registerHotKey(new Function("if (jsx3.EventHelp.isDragging()) jsx3.EventHelp.reset();"), Event.KEY_ESCAPE, false, false, false);
    // Register context-sensitive help system
    this._regHelpKey();

    //validate that the GUI container to place our app inside is not a null reference
    var objGUI = this.getEnv("GUIREF");

    if (objGUI) {
      //derive overflow settings for the outermost app container
      var strOverflow = "";
      var of = this.getEnv("OVERFLOW");
      if (of == Block.OVERFLOWSCROLL) {
        strOverflow = "overflow:auto;";
      } else if (of == Block.OVERFLOWHIDDEN) {
        strOverflow = "overflow:hidden;";
      }

      //derive position settings
      var strPosition, strLeft = "", strTop = "";
      if (this.getEnv("POSITION") == 0) {
        strPosition = "relative";
      } else {
        strPosition = "absolute";
        strLeft = "left:" + this.getEnv("LEFT") + ";";
        strTop = "top:" + this.getEnv("TOP") + ";";
      }
      var strWidth = "width:" + this.getEnv("WIDTH") + ";";
      var strHeight = "height:" + this.getEnv("HEIGHT") + ";";

      //generate initial application container
      objGUI.innerHTML = '<div id="' + Server._DIVID + '" style="position:' + strPosition + ';' + strOverflow +
          strLeft + strTop + strWidth + strHeight + '"></div>';
      objGUI = objGUI.lastChild;

      //initialize the box profile for root (this is how the box profiling is first initiated); basically pass the drawspace that root should live within
      this.JSXROOT.syncBoxProfileSync({left:0,top:0,parentwidth:objGUI.clientWidth,parentheight:objGUI.clientHeight});

      if (objGUI)
        objGUI.innerHTML = this.JSXROOT.paint();
    }

    try {
      //call _doLoad (regardless of whether or not there is a view component for this server instance, still try to initialize the environment
      this._doLoad(objXML);
    } catch (e) {
      LOG.fatal(jsx3._msg("serv.err_paint", this), jsx3.NativeError.wrap(e));
    }

    try {
      //initialize the on load script now that all files, includes, and object sets have been loaded
      this.eval(this.getEnv("onload"));
    } catch (e) {
      LOG.fatal(jsx3._msg("serv.err_onload", this), jsx3.NativeError.wrap(e));
    }
  };

  
  Server_prototype._regHelpKey = function() {
    var helpKey = this.getDynamicProperty("jsx3.app.Server.help." + jsx3.app.Browser.getSystem()) ||
                  this.getDynamicProperty("jsx3.app.Server.help");
    if (helpKey)
      this.registerHotKey(jsx3.gui.HotKey.valueOf(helpKey, jsx3.makeCallback("_onHelpKey", this)));
  };

  
  Server_prototype._doLoad = function(objXML) {
    var child = null;

    if (objXML) {
      child = this.JSXBODY.loadXML(objXML, true);
    } else {
      var strComponentURL = this.getEnv("objectseturl");

      if (strComponentURL)
        child = this.JSXBODY.load(strComponentURL, true);
    }

    if (child)
      child.setPersistence(jsx3.app.Model.PERSISTEMBED);
  };

  
  Server_prototype.setDimensions = function(left, top, width, height) {
    //convert array to ints
    if (left instanceof Array) {
      top = left[1];
      width = left[2];
      height = left[3];
      left = left[0];
    }

    var objGUI = this.JSXROOT.getRendered();
    if (objGUI) {
      if (left) objGUI.parentNode.style.left = left + "px";
      if (top) objGUI.parentNode.style.top = top + "px";
      if (width) objGUI.parentNode.style.width = width + "px";
      if (height) objGUI.parentNode.style.height = height + "px";
    }
  };

  
  Server_prototype.loadInclude = function(strSrc, strId, strType, bReload) {
    //resolve id to an empty string if not passed
    if (strId == null) strId = "";

    //create query string to force the browser to reload this resource from the remote server unless set to never reload
    var strAppend = (bReload) ? ((((strSrc+"").indexOf("?") == -1)?"?":"&") + (new Date()).valueOf()) : "";

    //branch based on file type
    if (strType == "css") {
      jsx3.CLASS_LOADER.loadResource(strSrc + strAppend, strId, strType);
    } else if (strType == "jss" || (strType == "ljss" && !jsx3.app.PropsBundle)) {
      var jssDoc = this.Cache.openDocument(strSrc, strId);

      if (jssDoc.hasError()) {
        jsx3.util.Logger.GLOBAL.error(jsx3._msg("serv.err_jss", + strSrc, jssDoc.getError()));
      } else {
        this.getProperties().loadXML(jssDoc, strId);
      }
    } else if (strType == "ljss") {
      var objCache = this.getCache();
      if (bReload) {
        var p = this.LJSS.getParents();
        for (var i = 0; i < p.length; i++) {
          if (p[i].getPath() == strSrc)
            this.LJSS.removeParent(p[i]);
        }
        
        jsx3.app.PropsBundle.clearCache(strSrc, objCache);
      }

      this.LJSS.addParent(jsx3.app.PropsBundle.getProps(strSrc, this.getLocale(), objCache));
    } else if (strType == "xml" || strType == "xsl") {
      return this.Cache.openDocument(strSrc, strId);
    } else if (strType == "script") {
      //try to remove existing script
      this.unloadInclude(strId);
      jsx3.CLASS_LOADER.loadResource(strSrc + strAppend, strId, strType);
    } else if (strType == "services") {
      jsx3.require("jsx3.net.Service");
      return (new jsx3.net.Service(strSrc)).setNamespace(this);
    } else {
      throw new jsx3.IllegalArgumentException('strType', strType);
    }
  };

  
  Server_prototype.unloadInclude = function(strId) {
    var objInclude = this.getRootDocument().getElementById(strId);
    try {
      if (objInclude) objInclude.parentNode.removeChild(objInclude);
    } catch (e) {
      LOG.warn(jsx3._msg("serv.err_unload", strId, this), jsx3.NativeError.wrap(e));
    }
  };

  
  Server_prototype.loadResource = function(strId) {
    var objSettings = this.getSettings();
    var includes = jsx3.util.List.wrap(objSettings.get('includes')).filter(function(x) { return x.id == strId; }).toArray(true);

    var lastInclude;
    for (var i = 0; i < includes.length; i++) {
      var include = includes[i];
      lastInclude = this.loadInclude(this.resolveURI(include.src), include.id, include.type);
    }

    if (includes.length == 0)
      LOG.warn(jsx3._msg("serv.err_badid", strId));

    return lastInclude;
  };

  
  Server_prototype.setDynamicProperty = function(strPropName, vntValue) {
    var jss = this.getProperties();
    jss.set(strPropName, vntValue);
  };

  
  Server_prototype.getDynamicProperty = function(strPropName, strToken) {
    var value = this.getProperties().get(strPropName);

    // jsx3.util.MessageFormat class is optional, should work without it
    if (arguments.length > 1 && jsx3.util.MessageFormat) {
      try {
        var format = new jsx3.util.MessageFormat(value);
        var args = new Array(arguments.length - 1);
        for (var i = 1; i < arguments.length; i++)
          args[i-1] = arguments[i];
        return format.format(args);
      } catch (e) {;}
    }

    return value;
  };

  
  Server_prototype.setCookie = function(name, value, expires, path, domain, secure, bRaw) {
    this.getRootDocument().cookie = name + "=" + (bRaw ? value : escape(value)) +
        ((expires) ? "; expires=" + expires.toGMTString() : "") +
        ((path) ? "; path=" + path : "") +
        ((domain) ? "; domain=" + domain : "") +
        ((secure) ? "; secure" : "");
  };

  
  Server_prototype.getCookie = function(name, bRaw) {
    var doc = this.getRootDocument();

    var dc = doc.cookie;
    var prefix = name + "=";
    var begin = dc.indexOf("; " + prefix);
    if (begin == -1) {
      begin = dc.indexOf(prefix);
      if (begin != 0) return null;
    } else {
      begin += 2;
    }

    var end = doc.cookie.indexOf(";", begin);
    if (end == -1) {
      end = dc.length;
    }

  var raw = dc.substring(begin + prefix.length, end);
    return bRaw ? raw : unescape(raw);
  };

  
  Server_prototype.deleteCookie = function(name, path, domain) {
    this.setCookie(name, "", new Date(1970,0,1), path, domain);
  };

  
  Server_prototype.getRootBlock = function() {
    if (this.JSXROOT == null) this._createRootAndBody();
    return this.JSXROOT;
  };

  
  Server_prototype.getAlertsParent = function() {
    return this.getRootBlock();
  };

  
  Server_prototype.getBodyBlock = function() {
    if (this.JSXROOT == null) this._createRootAndBody();
    return this.JSXBODY;
  };

  
  Server_prototype.getRootObjects = function() {
    return this.JSXBODY.getChildren().concat();
  };

  
  Server_prototype.getCache = function() {
    return this.Cache;
  };

  
  Server_prototype.getProperties = function() {
    return this.JSS;
  };

  
  Server_prototype.getDOM = function() {
    return this.DOM;
  };

  
  Server_prototype.getJSX = function(strId) {
    return this.DOM.get(strId);
  };

  
  Server_prototype.getJSXByName = function(strId) {
    return this.DOM.getByName(strId);
  };

  
  Server_prototype.getJSXById = function(strId) {
    return this.DOM.getById(strId);
  };

  Server_prototype.beep = function() {
    if (! jsx3.gui.WindowBar) return;

    var windowBar = this.JSXROOT.findDescendants(
      function(x) {return (x instanceof jsx3.gui.WindowBar)
          && x.getType() == jsx3.gui.WindowBar.TYPEMENU;}, false, false);
    if (windowBar != null)
      windowBar.beep();
  };

  
  Server_prototype.createAppWindow = function(strName) {
    jsx3.require("jsx3.gui.Window");

    if (this.WINDOWS == null)
      this.WINDOWS = this._createRoot("JSXWINDOWS");

    if (this.WINDOWS.getChild(strName) != null)
      throw new jsx3.IllegalArgumentException("strName", strName);

    var w = new jsx3.gui.Window(strName);
    this.WINDOWS.setChild(w);
    return w;
  };

  
  Server_prototype.loadAppWindow = function(strSource, objResolver) {
    jsx3.require("jsx3.gui.Window");

    if (this.WINDOWS == null)
      this.WINDOWS = this._createRoot("JSXWINDOWS");

    var component = null;
    if (strSource instanceof jsx3.xml.Entity)
      component = this.WINDOWS.loadXML(strSource, false, objResolver);
    else
      component = this.WINDOWS.load(strSource, false, objResolver);

    if (component == null)
      throw new jsx3.Exception(jsx3._msg("serv.win_err", strSource));
    if (!(component instanceof jsx3.gui.Window)) {
      component.getParent().removeChild(component);
      throw new jsx3.Exception(jsx3._msg("serv.win_notwin", strSource, component.getClass()));
    }

    if (this.WINDOWS.getChild(component.getName()) != component) {
      component.getParent().removeChild(component);
      throw new jsx3.Exception(jsx3._msg("serv.win_name", strSource, component.getName()));
    }

    return component;
  };

  
  Server_prototype.getAppWindow = function(strName) {
    if (this.WINDOWS != null)
      return this.WINDOWS.getChild(strName);
    return null;
  };

  
  Server_prototype.getDocumentOf = function(objJSX) {
    var parent = objJSX;
    while (parent != null) {
      if (jsx3.gui.Window && parent instanceof jsx3.gui.Window)
        return parent.getDocument();
      if (parent._jsxserver != null)
        return this.getRootDocument();

      parent = parent.getParent();
    }
    return this.getRootDocument();
  };

  Server_prototype.getRootDocument = function() {
    var rootGUI = this.getEnv('GUIREF');
    return rootGUI != null ? rootGUI.ownerDocument : null;
  };

  
  Server_prototype.getRenderedOf = function(objJSX) {
    var id = objJSX.getId();
    var rootDoc = this.getRootDocument();
    if (rootDoc == null) return null;

    var objGUI = rootDoc.getElementById(id);

    if (objGUI == null && this.WINDOWS != null) {
      var windows = this.WINDOWS.getChildren();
      for (var i = 0; objGUI == null && i < windows.length; i++) {
        var doc = windows[i].getDocument();
        if (doc)
          objGUI = doc.getElementById(id);
      }
    }

    return objGUI;
  };

  
  Server_prototype.registerHotKey = function(vntCallback, vntKey, bShift, bControl, bAlt) {
    return this.getRootBlock().registerHotKey(vntCallback, vntKey, bShift, bControl, bAlt);
  };

  
  Server_prototype.checkHotKeys = function(objEvent) {
    return this.getRootBlock().checkHotKeys(objEvent.event);
  };



  
  Server_prototype.getServer = function() {
    return this;
  };



  
  Server_prototype.isAtLeastVersion = function(strVersion) {
    return jsx3.util.compareVersions(this.getEnv('jsxversion'), strVersion) >= 0;
  };

  
  Server_prototype.resolveURI = function(strURI) {
    var uri = jsx3.net.URI.valueOf(strURI);

    if (this.isAtLeastVersion("3.2") && !URIResolver.isAbsoluteURI(uri)) {
      var appPathUri = this.getEnv("apppathuri");
      uri = appPathUri.resolve(uri);
    }
    
    return URIResolver.DEFAULT.resolveURI(uri);
  };

  
  Server_prototype.getUriPrefix = function() {
    return this.getEnv("apppathuri").toString();
  };

  
  Server_prototype.relativizeURI = function(strURI, bRel) {
    var loc = Browser.getLocation();
    var appPathUri = this.getEnv("apppathabs");
    var relative = appPathUri.relativize(loc.resolve(strURI));

    if (relative.isAbsolute() || bRel) {
      return relative;
    } else {
      var relPath = this.getEnv("apppathrel");
      if (relPath) {
        return jsx3.net.URI.fromParts("jsxapp", null, relPath.replace(/\//g, "!"), null,
            "/" + relative.getPath(), relative.getQuery(), relative.getFragment());
      } else {
        // special handling for GI_Builder/
        return jsx3.net.URI.fromParts(null, null, null, null,
            this.getEnv("apppath") + "/" + relative.getPath(), relative.getQuery(), relative.getFragment());
      }
    }
  };
      
  URIResolver.register("jsxapp", function(uri) {
    var host = uri.getHost();
    if (host) {
      var appPath = host.replace(/!/g, "/");
      var loadedApp = jsx3.System.getAppByPath(appPath);
      return loadedApp || new Server.Resolver(appPath);
    }
    return URIResolver.DEFAULT;
  });

  
  Server_prototype.getLocale = function() {
    if (this._locale == null)
      this._locale = this.getDefaultLocale();

    return this._locale != null ? this._locale : jsx3.System.getLocale();
  };

  
  Server_prototype.setLocale = function(objLocale) {
    
    this._locale = objLocale;
  };

  
  Server_prototype.getDefaultLocale = function() {
    var localeKey = this.getSettings().get("default_locale");
    if (localeKey == null) return null;
    localeKey = jsx3.util.strTrim(localeKey.toString());
    // Locale is optional
    return localeKey.length > 0 && jsx3.util.Locale ? jsx3.util.Locale.valueOf(localeKey) : null;
  };

  
  Server_prototype.reloadLocalizedResources = function() {
    var p = this.LJSS.getParents();
    this.LJSS.removeAllParents();
    
    for (var i = 0; i < p.length; i++) {
      var props = jsx3.app.PropsBundle.getProps(p[i].getPath(), this.getLocale(), this.getCache());
      this.LJSS.addParent(props);
    }
  };

  
  Server_prototype._onHelpKey = function(objEvent) {
    var objJSX = jsx3.html.getJSXParent(objEvent.srcElement(), this);
    if (!objJSX) objJSX = this.JSXROOT;
    return objJSX ? this.invokeHelp(objJSX) : false;
  };

  
  Server_prototype.invokeHelp = function(objJSX) {
    var strId = null;
    while (objJSX && !strId) {
      strId = objJSX.getHelpId();
      objJSX = objJSX.getParent();
    }

    if (strId)
      this.publish({subject:Server.HELP, helpid:strId, model:objJSX});

    return Boolean(strId);
  };
      
  
  Server_prototype.getAddins = function() {
    var a = [];
    var names = this.getSettings().get("addins");
    if (names) {
      for (var i = 0; i < names.length; i++) {
        var addin = jsx3.System.getAddin(names[i]);
        if (addin) a.push(addin);
      }
    }
    return a;
  };

  
  Server_prototype.toString = function() {
    return this.jsxsuper() + " " + this.getAppPath() + ":" + this.getEnv("namespace");
  };



  
  Server.getVersion = function() {
    return "3.00.00";
  };



});

// if Alerts is already loaded (shouldn't be), then go ahead and implement it here
if (jsx3.gui.Alerts)
  jsx3.app.Server.jsxclass.addInterface(jsx3.gui.Alerts.jsxclass);




jsx3.Server = jsx3.app.Server;




jsx3.Class.defineClass("jsx3.app.Server.Resolver", null, [jsx3.net.URIResolver], 
    function(ServerURIResolver, ServerURIResolver_prototype) {

  var URIResolver = jsx3.net.URIResolver;
      
  ServerURIResolver_prototype.init = function(strHost) {
    this._host = strHost;
    this._uri = new jsx3.net.URI(jsx3.getEnv("jsxhomepath") + jsx3.APP_DIR_NAME + "/" + strHost.replace(/!/g, "/") + "/");
  };
      
  ServerURIResolver_prototype.resolveURI = function(strURI) {
    var uri = jsx3.net.URI.valueOf(strURI);
    
    if (! URIResolver.isAbsoluteURI(uri))
      uri = URIResolver.DEFAULT.resolveURI(this._uri.resolve(uri));

    return URIResolver.DEFAULT.resolveURI(uri);
  };

  ServerURIResolver_prototype.getUriPrefix = function() {
    return this._uri.toString();
  };

  ServerURIResolver_prototype.relativizeURI = function(strURI, bRel) {
    var relative = this._uri.relativize(strURI);
    
    if (relative.isAbsolute() || bRel)
      return relative;
    else
      return jsx3.net.URI.fromParts("jsxapp", null, this._host, null, 
          "/" + relative.getPath(), relative.getQuery(), relative.getFragment());    
  };
  
  ServerURIResolver_prototype.toString = function() {
    return this._uri.toString();
  };

});

