


jsx3.Class.defineClass("jsx3.app.UserSettings", null, null, function(UserSettings, UserSettings_prototype) {
  
  var Entity = jsx3.xml.Entity;
  var LOG = jsx3.util.Logger.getLogger(UserSettings.jsxclass.getName());
  
  
  UserSettings.PERSIST_SESSION = 1;

  
  UserSettings.PERSIST_INDEFINITE = 2;
  
  
  UserSettings_prototype.server = null;
  
  UserSettings_prototype.persist = null;
  
  UserSettings_prototype.value = null;
  
  
  UserSettings_prototype.init = function(objServer, intPersistence) {
    if (intPersistence == null)
      intPersistence = UserSettings.PERSIST_INDEFINITE;
    
    this.server = objServer;
    this.persist = intPersistence;
    
    var cookie = this.getCookieValue();
    this.value = UserSettings.deserialize(cookie);
  };
  
  
  UserSettings_prototype.get = function(strKey) {
    var obj = this.value;
    
    for (var i = 0; i < arguments.length; i++) {
      if (typeof(obj) != 'object' || (obj instanceof Array))
        return null;
      
      obj = obj[arguments[i]];
    }
    
    return obj;
  };
  
  
  UserSettings_prototype.set = function(strKey, value) {
    var obj = this.value;
    
    for (var i = 0; i < arguments.length - 2; i++) {
      var child = obj[arguments[i]];
      
      if (typeof(child) != 'object' || (child instanceof Array)) {
        child = obj[arguments[i]] = null;
      }
      
      if (child == null) {
        child = obj[arguments[i]] = {};
      }
      
      obj = child;
    }
    
    obj[arguments[arguments.length - 2]] = arguments[arguments.length - 1];
    
    return obj;
  };
  
  
  UserSettings_prototype.remove = function(strKey) {
    var obj = this.value;
    
    for (var i = 0; i < arguments.length - 1; i++) {
      var child = obj[arguments[i]];
      
      if (child == null || typeof(child) != 'object' || (child instanceof Array))
        return;
        
      obj = child;
    }
    
    delete obj[arguments[arguments.length - 1]];
  };
  
  
  UserSettings_prototype.clear = function() {
    this.value = {};
    var name = this.getCookieName();
    var settings = this.server.getSettings();
    var domain = settings.get('user-settings','domain');
    var path = settings.get('user-settings','path');
    this.server.deleteCookie(name, path, domain);
  };

  
  UserSettings_prototype.save = function() {
    var value = UserSettings.serialize(this.value);
        
    // change '<' to '*' and '>' to '+'
    value = value.replace(/\*/g, '%2A');
    value = value.replace(/\+/g, '%2B');
    value = value.replace(/</g, '*');
    value = value.replace(/>/g, '+');
    value = escape(value);
    
    var bytes = value.length;
    if (bytes > 4096)
      LOG.warn(jsx3._msg("usrset.large", bytes));
    
    this.setCookieValue(value);
  };
  
  
  UserSettings_prototype.getCookieName = function() {
    return this.server.getEnv('NAMESPACE') + 
        (this.persist == UserSettings.PERSIST_SESSION ? "_ses" : "_ind");
  };
  
  
  UserSettings_prototype.getCookieValue = function() {
    var name = this.getCookieName();
    var value = this.server.getCookie(name, true);
    
    if (value) {
      // undo custom escaping
      value = value.replace(/\*/g, '<');
      value = value.replace(/\+/g, '>');
      value = unescape(value);
    }
    
    return value;
  };
  
  
  UserSettings_prototype.setCookieValue = function(strValue) {
    var name = this.getCookieName();
    var settings = this.server.getSettings();
    var domain = settings.get('user-settings','domain');
    var path = settings.get('user-settings','path');
    var today = new Date();
    var expires = (this.persist == UserSettings.PERSIST_SESSION) ? null : 
        new Date(today.getFullYear() + 1, today.getMonth(), today.getDate());
      
    this.server.setCookie(name, strValue, expires, path, domain, null, true);
  };
  
  
  UserSettings.deserialize = function(strValue) {
    if (! strValue) 
      return {};
    
    var doc = new jsx3.xml.Document();
    doc.loadXML(strValue);
    return UserSettings.deserializeValue(doc.getRootNode());
  };

  
  UserSettings.deserializeValue = function(objNode) {
    var name = objNode.getNodeName();
    
    if (name == 's') {
      return objNode.getValue();
    } else if (name == 'n') {
      return Number(objNode.getValue());
    } else if (name == 'm') {
      return objNode.getChildNodes().map(function(x){
        return [x.getAttribute('n'), UserSettings.deserializeValue(x)];
      }, false, true);
    } else if (name == 'a') {
      return objNode.getChildNodes().map(function(x){
        return UserSettings.deserializeValue(x);
      }).toArray(true);
    } else if (name == 'b') {
      return objNode.getValue() == "1";
    } else if (name == 'u') {
      return null;
    } else {
      LOG.warn(jsx3._msg("usrset.deser", name));
      return null;
    }
  };
  
  
  UserSettings.serialize = function(objValue) {
    var doc = new jsx3.xml.Document();
    doc.loadXML('<m/>');
    var node = doc.getRootNode();
    
    for (var f in objValue)
      UserSettings.serializeValue(objValue[f], f, node);

    return node.getXML();
  };
    
  
  UserSettings.serializeValue = function(objValue, strName, objParent) {
    var node = null;
    var type = typeof(objValue);
    
    if (objValue == null || type == "undefined") {
      node = objParent.createNode(Entity.TYPEELEMENT, "u");
    } else if (type == "string") {
      node = objParent.createNode(Entity.TYPEELEMENT, "s");
      node.setValue(objValue);
    } else if (type == "number") {
      node = objParent.createNode(Entity.TYPEELEMENT, "n");
      node.setValue(objValue);
    } else if (type == "boolean") {
      node = objParent.createNode(Entity.TYPEELEMENT, "b");
      node.setValue(objValue ? "1" : "0");
    } else if (type == "object") {
      if (objValue instanceof Array) {
        node = objParent.createNode(Entity.TYPEELEMENT, "a");
        for (var i = 0; i < objValue.length; i++) {
          UserSettings.serializeValue(objValue[i], i.toString(), node);
        }
      } else {
        node = objParent.createNode(Entity.TYPEELEMENT, "m");
        for (var f in objValue)
          UserSettings.serializeValue(objValue[f], f, node);
      }
    } else if (type == "function") {
      ;
    } else {
      throw new jsx3.Exception(jsx3._msg("usrset.persist", type));
    }
    
    node.setAttribute('n', strName);
    objParent.appendChild(node);
  };

  UserSettings_prototype.toString = function() {
    return this.jsxsuper() + this.server.getAppPath();
  };

});




jsx3.UserSettings = jsx3.app.UserSettings;


