

(function(jsx3) {

  var extend = function(target, source) {
    for (var f in source)
      target[f] = source[f];
    return target;
  };

  var json = function(o) {
    // TODO: proper escaping
    if (o == null) return "null";
    if (typeof(o) == "object") {
      if (o instanceof Array) {
        var s = [];
        for (var i = 0; i < o.length; i++)
          s.push(json(o[i]));
        return "[" + s.join(",") + "]";
      } else {
        var s = [];
        for (var f in o)
          if (o[f] != o.constructor.prototype[f])
            s.push(f + ":" + json(o[f]));
        return "{" + s.join(",") + "}";
      }
    } else if (typeof(o) == "string") {
      return "\"" + jsx3.util.strEscapeJSON(o) + "\"";
    } else {
      return o.toString();
    }
  };

  jsx3.$Object = {

    
    extend: function(o) {
      return extend(this, o);
    },

    
    clone: function() {
      var c = jsx3.$O();
      c.extend(this);
      return c;
    },

    json: function() {
      return json(this);
    }
  };

  jsx3.$O = function(o) {
    return extend(o || {}, jsx3.$Object);
  };

  jsx3.$Array = {

    
    each: function __Array_each(fct) {
      for (var i = 0; i < this.length; i++)
        fct(this[i]);
    },

    
    map: function(fct) {
      var a = jsx3.$A();
      for (var i = 0; i < this.length; i++)
        a[i] = fct(this[i]);
      return a;
    },

    
    filter: function(fct) {
      var a = jsx3.$A();
      for (var i = 0; i < this.length; i++)
        if (fct(this[i]))
          a.push(this[i]);
      return a;
    },

    
    indexOf: function(o) {
      for (var i = 0; i < this.length; i++)
        if (this[i] === o)
          return i;
      return -1;
    },

    
    contains: function(o) {
      return this.indexOf(o) >= 0;
    },

    
    remove: function(o) {
      var index = this.indexOf(o);
      if (index >= 0)
        return this.splice(index, 1)[0];
    },

    
    find: function(fct) {
      for (var i = 0; i < this.length; i++)
        if (fct(this[i]))
          return this[i];
    },

    
    unique: function() {
      var a = this.concat();
      for (var i = a.length - 1; i >= 1; i--) {
        for (var j = i - 1; j >= 0; j--) {
          if (a[i] === a[j]) {
            a.splice(i, 1);
            break;
          }
        }
      }
      return jsx3.$A(a);
    },

    clone: function() {
      return jsx3.$A(this.concat());
    }
  };

  jsx3.$Hash = jsx3.$Object.clone().extend({

    
    each: function(fct) {
      for (var f in this) {
        if (this[f] != this.constructor.prototype[f] && this[f] != jsx3.$Hash[f])
          fct(f, this[f]);
      }
    },

    
    keys: function() {
      var a = [];
      for (var f in this) {
        if (this[f] != this.constructor.prototype[f] && this[f] != jsx3.$Hash[f])
          a.push(f);
      }
      return jsx3.$A(a);
    },

    
    values: function() {
      var a = [];
      for (var f in this) {
        if (this[f] != this.constructor.prototype[f] && this[f] != jsx3.$Hash[f])
          a.push(this[f]);
      }
      return jsx3.$A(a);
    }
  });

  jsx3.$Function = {
    bind: function(thisObj, argsArr) {
      var fct = this;

      if (argsArr == null || argsArr.length == 0) {
        return jsx3.$F(function __boundFunction() {
	  document.body;  // fall off trace
          return fct.apply(thisObj, arguments);
        });
      } else {
        return function() {
          var myArgs;

          if (arguments.length > 0) {
            myArgs = [];
            for (var i = 0; i < argsArr.length; i++)
              myArgs.push(argsArr[i]);
            for (var i = 0; i < arguments.length; i++)
              myArgs.push(arguments[i]);
          } else {
            myArgs = argsArr;
          }

          return fct.apply(thisObj, myArgs);
        };
      }
    },
    
    throttled: function() {
      var fct = this;
      return jsx3.$F(function() {
        if (fct._thlto)
          window.clearTimeout(fct._thlto);
        fct._thlto = window.setTimeout(jsx3.$F(fct).bind(this, arguments));
      });
    },

    slept: function() {
      var fct = this;
      return jsx3.$F(function() {
        window.setTimeout(fct.bind(this, arguments));
      });
    }
  };

  jsx3.$String = jsx3.$Object.clone().extend({

    
    endsWith: function(s) {
      return this.lastIndexOf(s) == this.length - s.length;
    },

    
    trim: function() {
      return jsx3.$S(this.replace(/(^\s*)|(\s*$)/g, ""));
    }
  });

  

  jsx3.$F = function(f) {
    return extend(f, jsx3.$Function);
  };

  var fixAsyncArg = jsx3.$F(function(args, n, arg) { args[n] = arg.rv(); });
  var handleAsyncArgs = function(args) {
    var c = null;

    for (var i = 0; i < args.length; i++) {
      var a = args[i];
      if (a instanceof AsyncRV) {
        if (a._done) {
          args[i] = a.rv();
        } else {
          a.when(fixAsyncArg.bind(null, [args, i, a]));
          c = c ? c.and(a) : a;
        }
      }
    }

    return c;
  };

  jsx3.$AsyncCB = function(objThis, arrArgs) {
    
    this._this = objThis;
    
    this._args = arrArgs;
    
    this._condition = handleAsyncArgs(arrArgs);
  };
  extend(jsx3.$AsyncCB.prototype, {

    
    _whenArg: jsx3.$F(function(n, arg) { this._args[n] = arg.rv(); }),
    
    
    done: function __AsyncCB_prototype_done(rv) {
      this._done = true;
      this._rv = rv;
      if (this._ondone)
        this._ondone(rv);
    },

    
    args: function() {
      return this._args;
    }

  });

  var AsyncRV = jsx3.$AsyncRV = function() {
  };
  extend(AsyncRV.prototype, {
    
    _trigger: function __AsyncRV_prototype_trigger(rv) {
      
      this._done = true;
      
      this._rv = rv;
      if (this._sub) {
        this._sub.each(function __AsyncRV_prototype_trigger_each_sub(e) { e(rv); });
        delete this._sub;
      }
    },

    
    rv: function() {
      if (!this._done)
        throw new Error("May not call $AsyncRV.rv() before the return value is set.");
      return this._rv;
    },

    
    when: function(cb, rv) {
      var fct = null;

      if (typeof(cb) == "function")
        fct = cb;
      else if (cb instanceof jsx3.$AsyncCB) {
        if (arguments.length > 1)
          fct = function() {cb.done(rv);};
        else
          fct = function __AsyncRV_prototype_when_lambda(chainedRV) {cb.done(chainedRV);};
      } else
        throw new Error();

      if (this._done) {
        fct(this._rv);
      } else {
        if (!this._sub)
          
          this._sub = jsx3.$A();
        this._sub.push(fct);
      }
    },

    
    and: function(rv) {
      var a = [this];
      for (var i = 0; i < arguments.length; i++)
        a.push(arguments[i]);
      return new AndRV(a);
    },

    
    or: function(rv) {
      var a = [this];
      for (var i = 0; i < arguments.length; i++)
        a.push(arguments[i]);
      return new OrRV(a);
    }
  });

  var MethodRV = function(cb) {
    
    this._cb = cb;
    
    cb._ondone = this._cbdone.bind(this);
  };
  MethodRV.prototype = new AsyncRV();
  extend(MethodRV.prototype, {
    
    _cbdone: jsx3.$F(function __MethodRV_prototype_cbdone(rv) {
      delete this._cb._ondone;
      this._trigger(rv);
    })
  });

  var AndRV = function(rvs) {
    
    this._ct = rvs.length;
    
    this._donect = 0;
    jsx3.$A(rvs).each(jsx3.$F(function(e) {
      if (e._done)
        this._donect++;
      else
        e.when(this._inc.bind(this));
    }).bind(this));

    if (this._ct == this._donect)
      this._trigger();
  };
  AndRV.prototype = new AsyncRV();
  extend(AndRV.prototype, {
    
    _inc: jsx3.$F(function __AndRV_prototype_inc() {
      this._donect++;
      if (this._donect == this._ct)
        this._trigger();
    })
  });

  var OrRV = function(rvs) {
    jsx3.$A(rvs).each(jsx3.$F(function(e) {
      if (e._done)
        this._inc();
      else
        e.when(this._inc.bind(this));
    }).bind(this));
  };
  OrRV.prototype = new AsyncRV();
  extend(OrRV.prototype, {
    
    _inc: jsx3.$F(function() {
      if (!this._done)
        this._trigger();
    })
  });

  jsx3.$A = function(a) {
    if (a == null)
      a = [];
    else if (typeof(a) == "object") {
      if (a instanceof Array) {

      } else if (typeof(a.length) == "number") {
        // works on arguments array
        var t = [];
        for (var i = 0; i < a.length; i++)
          t[i] = a[i];
        a = t;
      } else {
        a = [a];
      }
    } else
      a = [a];
    return extend(a, jsx3.$Array);
  };

  jsx3.$H = function(o) {
    if (o && typeof(o) == "object" && typeof(o.length) != "undefined") {
      var h = {};
      for (var i = 0; i < o.length; i++)
        h[o[i]] = 1;
      return extend(h, jsx3.$Hash)
    } else {
      return extend(o || {}, jsx3.$Hash);
    }
  };

  jsx3.$S = function(s) {
    if (s == null) return s;
    return extend(s, jsx3.$String);
  };

  jsx3.$Y = function(f) {
    return function() {
      var cb = new jsx3.$AsyncCB(this, arguments);
      var rv = new MethodRV(cb);

//      var stack = (jsx3.lang && jsx3.lang.getStack) ? jsx3.lang.getStack() : [];
//      window.setTimeout(jsx3.$F(function() {
//        if (!cb._done) {
//          jsx3.log("Never finished: " + this + " " + f + "\nargs:" + jsx3.Method.argsAsArray(cb.args()) + "\n" +
//                   jsx3.Exception.formatStack(stack));
//        }
//      }).bind(this), 10000);

      if (cb._condition) {
        var objThis = this;
        cb._condition.when(function() {
          var r = f.apply(objThis, [cb]);
          if (r instanceof AsyncRV)
            r.when(cb);
        });
      } else {
        var r = f.apply(this, [cb]);
        if (r instanceof AsyncRV)
          r.when(cb);
      }

      return rv;
    };
  };

  jsx3.$Z = function(strMethod, objThis) {
    if (objThis instanceof AsyncRV)
      objThis.when(function(rv) { objThis = rv; });

    return function() {
      var rv = new AsyncRV();
      var a = jsx3.Method.argsAsArray(arguments);

      if (objThis instanceof AsyncRV) {
        objThis.when(function() {
          onceDone(objThis, strMethod, a, rv);
        });
      } else {
        onceDone(objThis || this, strMethod, a, rv);
      }

      return rv;
    }
  };

  var onceDone = function(objThis, strMethod, args, rv) {
    var fct = typeof(strMethod) == "function" ? strMethod : objThis[strMethod];
    var c = handleAsyncArgs(args);
    if (c) {
      c.when(function __$Z_onceDone_lambda() {
        rv._trigger(fct.apply(objThis, args))
      });
    } else {
      rv._trigger(fct.apply(objThis, args));
    }
  };

})(jsx3);
