


jsx3.Class.defineClass('jsx3.util.List', null, null, function(List, List_prototype) {

  
  List.wrap = function(a) {
    if (a instanceof List) {
      return a;
    } else if (a instanceof Array) {
      return new List(a, true);
    } else {
      throw new jsx3.IllegalArgumentException("a", a);
    }
  };

  
  List_prototype.init = function(a, bLive) {
    if (typeof(a) == "number") {
      this._src = new Array(a);
    } else if (a instanceof List) {
      this._src = a._src.concat();
    } else {
      a = List._convertArrayLikeToArray(a);
      if (a instanceof Array) {
        this._src = bLive ? a : a.concat();
      } else {
        
        this._src = [];
      }
    }


    
    this._iterator = -1;

  };

  
  List_prototype.size = function() {
    return this._src.length;
  };

  
  List_prototype.get = function(intIndex) {
    return this._src[intIndex];
  };

  
  List_prototype.set = function(intIndex, objElm) {
    this._src[intIndex] = objElm;
  };

  
  List_prototype.iterator = function() {
    return new List.Iterator(this);
  };

  
  List_prototype.clear = function() {
    this._src.splice(0, this._src.length);
  };

  
  List_prototype.indexOf = function(objElm, intStartAt) {
    if (intStartAt == null) intStartAt = 0;
    var size = this.size();
    for (var i = intStartAt; i < size; i++) {
      if (this.get(i) === objElm)
        return i;
    }
    return -1;
  };

  
  List_prototype.lastIndexOf = function(objElm, intStartAt) {
    if (intStartAt == null) intStartAt = this.size() - 1;
    for (var i = intStartAt; i >= 0; i--) {
      if (this.get(i) === objElm)
        return i;
    }
    return -1;
  };

  
  List_prototype.contains = function(objElm) {
    return this.indexOf(objElm) >= 0;
  };

  
  List_prototype.remove = function(objElm) {
    var index = this.indexOf(objElm);
    if (index >= 0)
      return this._src.splice(index, 1)[0];
    return null;
  };

  
  List_prototype.removeAt = function(intStart, intEnd) {
    if (arguments.length == 2) {
      return List.wrap(this._src.splice(intStart, (intEnd - intStart)));
    } else {
      return this._src.splice(intStart, 1)[0];
    }
  };

  
  List_prototype.clone = function() {
    return new List(this);
  };

  
  List_prototype.add = function(objElm, intAt) {
    var src = this._src;
    if (intAt == null)
      src[src.length] = objElm;
    else
      src.splice(intAt, 0, objElm);
  };

  
  List_prototype.addAll = function(a, intAt) {
    if (a instanceof List)
      a = a.toArray(true);
    else
      a = List._convertArrayLikeToArray(a);

    if (a instanceof Array) {
      if (intAt == null)
        this._src.push.apply(this._src, a);
      else
        this._src = this._src.slice(0, intAt).concat(a, this._src.slice(intAt));
    } else {
      throw new jsx3.IllegalArgumentException("a", a);
    }
  };

  
  List._convertArrayLikeToArray = function(a) {
    if (a == null || a instanceof Array) {
      return a;
    } else if (typeof(a.length) == "number") {
      var c = new Array(a.length);
      for (var i = 0; i < a.length; i++)
        c[i] = a[i];
      return c;
    } else {
      return a;
    }
  };

  
  List_prototype.slice = function(intStart, intEnd) {
    // NOTE: Fx and IE behave differently even if null is sent as the second parameter
    return List.wrap(arguments.length > 1 ? this._src.slice(intStart, intEnd) : this._src.slice(intStart));
  };

  
  List_prototype.sort = function(fctComparator) {
    // NOTE: Fx and IE behave differently even if null is sent as the first parameter
    if (fctComparator) this._src.sort(fctComparator);
    else this._src.sort();
  };

  
  List_prototype.toArray = function(bLive) {
    return bLive ? this._src : this._src.concat();
  };

  
  List_prototype.equals = function(l) {
    if (this === l) return true;
    if (!(l instanceof List)) return false;
    var size = this.size();
    if (size != l.size()) return false;
    for (var i = 0; i < size; i++) {
      if (this.get(i) !== l.get(i)) return false;
    }
    return true;
  };

  
  List_prototype.filter = function(fctFilter) {
    var filtered = [];
    var size = this.size();
    for (var i = 0; i < size; i++) {
      var item = this.get(i);
      if (fctFilter(item))
        filtered[filtered.length] = item;
    }
    return List.wrap(filtered);
  };

  
  List_prototype.map = function(fctMapper, bExpand, bObject) {
    var size = this.size();
    if (bExpand) {
      if (bObject) {
        var mapped = {};
        for (var i = 0; i < size; i++) {
          var pairs = fctMapper(this.get(i));
          for (var j = 0; j < pairs.length; j+=2)
            mapped[pairs[j]] = pairs[j+1];
        }
        return mapped;
      } else {
        var mapped = [];
        for (var i = 0; i < size; i++) {
          var val = fctMapper(this.get(i));
          if (val instanceof Array)
            mapped.push.apply(mapped, val);
          else
            mapped[mapped.length] = val;
        }
        return List.wrap(mapped);
      }
    } else {
      if (bObject) {
        var mapped = {};
        for (var i = 0; i < size; i++) {
          var pair = fctMapper(this.get(i));
          mapped[pair[0]] = pair[1];
        }
        return mapped;
      } else {
        var mapped = new Array(size);
        for (var i = 0; i < size; i++)
          mapped[i] = fctMapper(this.get(i));
        return List.wrap(mapped);
      }
    }
  };

  List_prototype.toString = function() {
    return "[" + this._src + "]";
  };



  
  List_prototype.reset = function() {
    this._iterator = -1;
    return this;
  };

  
  List_prototype.next = function() {
    return this.get(++this._iterator);
  };

  
  List_prototype.move = function(intIndex) {
    this._iterator = intIndex;
    return this;
  };

  
  List_prototype.hasNext = function() {
    return this._iterator < (this.size() - 1);
  };

  
  List_prototype.getIndex = function() {
    return this._iterator;
  };

  
  List_prototype.getItem = function(intIndex) {
    return this.get(intIndex);
  };

  
  List_prototype.getLength = function() {
    return this.size();
  };



});


jsx3.Class.defineInterface('jsx3.util.Iterator', null, function(Iterator, Iterator_prototype) {

  
  Iterator_prototype.next = jsx3.Method.newAbstract();

  
  Iterator_prototype.hasNext = jsx3.Method.newAbstract();

  
  Iterator_prototype.remove = jsx3.Method.newAbstract();

});

// @jsxobf-clobber-shared  _list _index _size


jsx3.Class.defineClass('jsx3.util.List.Iterator', null, [jsx3.util.Iterator], function(Iterator, Iterator_prototype) {

  Iterator_prototype.init = function(objList) {
    this._list = objList;
    this._size = objList ? objList.size() : 0;
    this._index = 0;
  };

  Iterator_prototype.next = function() {
    return this._list.get(this._index++);
  };

  Iterator_prototype.hasNext = function() {
    return this._index < this._size;
  };

  Iterator_prototype.remove = function() {
    if (this._index > 0) {
      this._list.removeAt(--this._index);
      this._size--;
    }
  };

});
