


jsx3.Class.defineClass('jsx3.util.Logger', null, null, function(Logger, Logger_prototype) {
  
  var Method = jsx3.Method;
  var Exception = jsx3.Exception;
  
  
  Logger.OFF = 0;
  
  
  Logger.FATAL = 1;
  
  
  Logger.ERROR = 2;
  
  
  Logger.WARN = 3;
  
  
  Logger.INFO = 4;
  
  
  Logger.DEBUG = 5;
  
  
  Logger.TRACE = 6;

  
  Logger.MAX_LEVEL = Logger.FATAL;
  
  Logger.MIN_LEVEL = Logger.TRACE;
  
  
  Logger.GLOBAL = null;
  
  
  Logger.getLogger = function(strName) {
    var manager = Logger.Manager.getManager();
    if (manager == null) return null;

    var logger = manager.getLogger(strName);
    if (logger == null) {
      logger = new Logger(strName);
      manager.addLogger(logger);
    }
    return logger;
  };
  
  
  Logger._LEVELS = [null, "FATAL", "ERROR", "WARN", "INFO", "DEBUG", "TRACE"];
  
  
  Logger.levelAsString = function(intLevel) {
    return Logger._LEVELS[intLevel] || "";
  };
  
  
  Logger_prototype._name = null;
  
  Logger_prototype._handlers = null;
  
  Logger_prototype._mylevel = null;
  
  Logger_prototype._level = Logger.INFO;
  
  Logger_prototype._parent = null;
  
  Logger_prototype._useParent = true;
  
  Logger_prototype._children = null;
  
  
  Logger_prototype.init = function(strName) {
    this._name = strName;
  };
  
  // bean methods
  
  
  Logger_prototype.getName = function() {
    return this._name;
  };
  
  
  Logger_prototype.addHandler = function(objHandler) {
    if (!this._handlers)
      this._handlers = jsx3.$A();
    this._handlers.push(objHandler);
  };
  
  
  Logger_prototype.removeHandler = function(objHandler) {
    if (this._handlers)
      this._handlers.remove(objHandler);
  };

  
  Logger_prototype.getLevel = function() {
    return this._mylevel;
  };
  
  
  Logger_prototype.getEffectiveLevel = function() {
    return this._level;
  };
  
  
  Logger_prototype.setLevel = function(intLevel) {
    intLevel = Math.max(Logger.OFF, Math.min(Logger.MIN_LEVEL, intLevel));
    this._mylevel = intLevel;
    this._updateLevel();
  };

  
  Logger_prototype._updateLevel = function() {
    var newValue = null;
    
    if (this._mylevel != null) {
      newValue = this._mylevel;
    } else if (this._parent != null) {
      newValue = this._parent._level;
    } else {
      newValue = Logger_prototype._level;
    }
    
    if (newValue != this._level) {
      this._level = newValue;
      if (this._children) {
        for (var i = 0; i < this._children.length; i++)
          this._children[i]._updateLevel();
      }
    }
  };
  
  
  Logger_prototype.getParent = function() {
    return this._parent;
  };
  
  
  Logger_prototype.setParent = function(objParent) {
    // remove from old parent's children
    if (this._parent != null)
      this._parent._children.remove(this);
    
    this._parent = objParent;
    // add to new parent's children
    if (this._parent != null) {
      if (!this._parent._children)
        this._parent._children = jsx3.$A();
      this._parent._children.push(this);
    }
    
    this._updateLevel();
  };
  
  
  Logger_prototype.getUseParentHandlers = function() {
    return this._useParent;
  };
  
  
  Logger_prototype.setUseParentHandlers = function(bUseParent) {
    this._useParent = bUseParent;
  };
    
  // log methods

  
  Logger_prototype._dispatch = function(objRecord) {
    var logger = this;
    var recordLevel = objRecord.getLevel();
    
    while (logger) {
      var handlers = logger._handlers;
      if (handlers) {
        for (var i = 0; i < handlers.length; i++) {
          // check handler level here instead of in Handler.handle() method
          var handler = handlers[i];
          if (handler.isLoggable(recordLevel)) {
            try {
              handler.handle(objRecord);
            } catch (e) {
              e = jsx3.NativeError.wrap(e);
              Logger.getLogger(Logger.jsxclass.getName()).error(jsx3._msg("logr.err_hand", handler.getName(), e), e);
            }
          }
        }
      }
      
      if (! logger.getUseParentHandlers())
        break;
      logger = logger.getParent();
    }
  };
  
  
  Logger_prototype.log = function(intLevel, strMessage, strArgs) {
    intLevel = Math.max(intLevel, Logger.MAX_LEVEL);
    if (this._level < intLevel) return;
    
    var arrParams = strArgs instanceof Array ? strArgs : Method.argsAsArray(arguments, 2);
    var record = new Logger.Record(strMessage, arrParams, intLevel, this.getName(), jsx3.lang.getCaller(1), null);
    this._dispatch(record);
  };
  
  
  Logger_prototype.logError = function(intLevel, strMessage, objError) {
    intLevel = Math.max(intLevel, Logger.MAX_LEVEL);
    if (this._level < intLevel) return;

    var record = new Logger.Record(strMessage, null, intLevel, this.getName(), jsx3.lang.getCaller(1), objError);
    this._dispatch(record);
  };
  
  
  Logger_prototype.logStack = function(intLevel, strMessage, intSkip) {
    intLevel = Math.max(intLevel, Logger.MAX_LEVEL);
    if (this._level < intLevel) return;
    
    var record = new Logger.Record(strMessage, null, intLevel, this.getName(), 
        jsx3.lang.getStack(intSkip != null ? intSkip : 0), null);
    this._dispatch(record);
  };
  
  
  Logger_prototype.isLoggable = function(intLevel) {
    intLevel = Math.max(intLevel, Logger.MAX_LEVEL);
    return this._level >= intLevel;
  };
  
  
  Logger_prototype.fatal = function(strMessage, strArgs) {
    if (strArgs == null || strArgs instanceof Array)
      this.log(Logger.FATAL, strMessage, strArgs);
    else if (strArgs instanceof Exception)
      this.logError(Logger.FATAL, strMessage, strArgs);
    else if (this._level >= Logger.FATAL)
      this.log(Logger.FATAL, strMessage, Method.argsAsArray(arguments, 1));
  };
  
  
  Logger_prototype.error = function(strMessage, strArgs) {
    if (strArgs == null || strArgs instanceof Array)
      this.log(Logger.ERROR, strMessage, strArgs);
    else if (strArgs instanceof Exception)
      this.logError(Logger.ERROR, strMessage, strArgs);
    else if (this._level >= Logger.ERROR)
      this.log(Logger.ERROR, strMessage, Method.argsAsArray(arguments, 1));
  };
  
  
  Logger_prototype.warn = function(strMessage, strArgs) {
    if (strArgs == null || strArgs instanceof Array)
      this.log(Logger.WARN, strMessage, strArgs);
    else if (strArgs instanceof Exception)
      this.logError(Logger.WARN, strMessage, strArgs);
    else if (this._level >= Logger.WARN)
      this.log(Logger.WARN, strMessage, Method.argsAsArray(arguments, 1));
  };
  
  
  Logger_prototype.info = function(strMessage, strArgs) {
    if (strArgs == null || strArgs instanceof Array)
      this.log(Logger.INFO, strMessage, strArgs);
    else if (strArgs instanceof Exception)
      this.logError(Logger.INFO, strMessage, strArgs);
    else if (this._level >= Logger.INFO)
      this.log(Logger.INFO, strMessage, Method.argsAsArray(arguments, 1));
  };
  
  
  Logger_prototype.debug = function(strMessage, strArgs) {
    if (strArgs == null || strArgs instanceof Array)
      this.log(Logger.DEBUG, strMessage, strArgs);
    else if (strArgs instanceof Exception)
      this.logError(Logger.DEBUG, strMessage, strArgs);
    else if (this._level >= Logger.DEBUG)
      this.log(Logger.DEBUG, strMessage, Method.argsAsArray(arguments, 1));
  };
  
  
  Logger_prototype.trace = function(strMessage, strArgs) {
    if (strArgs == null || strArgs instanceof Array)
      this.log(Logger.TRACE, strMessage, strArgs);
    else if (strArgs instanceof Exception)
      this.logError(Logger.TRACE, strMessage, strArgs);
    else if (this._level >= Logger.TRACE)
      this.log(Logger.TRACE, strMessage, Method.argsAsArray(arguments, 1));
  };

  Logger_prototype.toString = function() {
    return this.jsxsuper() + " " + this.getName();
  };
 


  // DEPRECATED: remove later

  
  Logger.reset = function() {
  };

  
  Logger.doLog = function(strErrorNumber, strMessage, PRIORITY, bTrace) {
    //if no priority is set, make this a low priority
    if (PRIORITY == null) PRIORITY = Logger.INFO;
    else if (PRIORITY < Logger.INFO) PRIORITY = Logger.INFO;
    else PRIORITY = Logger.DEBUG;

    // QUESTION: should we somehow notify that the message has not gone to out?
    if (Logger.GLOBAL) {
      var message = strMessage != null ? "(" + strErrorNumber + ") " + strMessage : strErrorNumber;
      if (bTrace || bTrace == null)
        Logger.GLOBAL.logStack(PRIORITY, message, 1);
      else
        Logger.GLOBAL.log(PRIORITY, message);
    }
  };

  
  Logger.logError = function(e, PRIORITY) {
    var message = "";
    for (var f in e) {
      if (message) message += " ";
      message += f + ":" + e[f];
    }
    Logger.doLog("ERRO01", message, PRIORITY, false);
  };

  
  Logger.getMinPriority = function() {
    return 3;
  };

  
  Logger.getLog = function() {
    return [];
  };

  
  Logger.errorToString = function(error) {
    var s = "";
    s+= "TIME: " + new Date(error.timestamp) + "\n";
    s+= "CODE: " + error.code + "\n";
    s+= "DESC: " + error.description + "\n";
    return s;
  };

  
  Logger.toString = function() {
    return "";
  };


  
});


jsx3.Class.defineClass('jsx3.util.Logger.Manager', null, null, function(Manager, Manager_prototype) {

  var Exception = jsx3.Exception;
  var Logger = jsx3.util.Logger;
  
  
  Manager.DEFAULT_CONFIG_FILE = "jsx:/../logger.xml";
  
  Manager.ROOT_KEY = "global";
  
  Manager.INSTANCE = null;
  
  Manager.LAZY_HANDLER = -1;
  
  
  Manager_prototype.initialize = function(objXML) {
    if (!objXML) {
      objXML = new jsx3.xml.Document();
      objXML.load(jsx3.getEnv("jsx_logger_config") || Manager.DEFAULT_CONFIG_FILE);
    }

    // check error, causes a window.alert() but lets the program continue
    if (objXML.hasError()) {
      window.alert(jsx3._msg("logr.err_conf", objXML.getError(), jsx3.resolveURI(objXML.getSourceURL())));
      objXML.loadXML("<configuration/>");
    }

    this._config = objXML;

    this._initHandlers();

    for (var f in this._loggers)
      this.addLogger(this._loggers[f]);
  };
  
  
  Manager.getManager = function() {
    if (Manager.INSTANCE == null) {
      Manager.INSTANCE = new Manager();
      Logger.GLOBAL = new Logger(Manager.ROOT_KEY);
      Manager.INSTANCE.addLogger(Logger.GLOBAL);
    }
    return Manager.INSTANCE;
  };
  
  
  Manager_prototype._config = null;
  
  Manager_prototype._loggers = null;
  
  Manager_prototype._handlers = null;
  
  
  Manager_prototype.init = function(objXML) {
    this._loggers = {};
    this._handlers = {};
  };

  
  Manager_prototype._handlerClassDidLoad = function(objClass) {
    var loaded = this._initHandlers("[@lazy='true' and @class='" + objClass.getName() + "']");
    this._wireNewHandlers(loaded);
  };

  
  Manager_prototype._wireNewHandlers = function(handlerNames) {
    for (var i = 0; i < handlerNames.length; i++) {
      var handlerName = handlerNames[i];
      var handler = this.getHandler(handlerName);

      var j = this._config.selectNodeIterator("/configuration/logger[handler-ref/@name='" + handlerName + "']");
      while (j.hasNext()) {
        var node = j.next();
        var loggerName = node.getAttribute('name');
        var logger = this.getLogger(loggerName);
        if (logger != null) {
          logger.addHandler(handler);
        }
      }
    }
  };
 
  
  Manager_prototype._initHandlers = function(strClause) {
    var initedNames = [];
    if (! this._config) return initedNames;

    var i = this._config.selectNodeIterator("/configuration/handler" + (strClause != null ? strClause : ""));

    var bNeedsJob = this._classesToLoad == null;

    while (i.hasNext()) {
      var node = i.next();
      var strName = node.getAttribute("name");
      if (this.getHandler(strName) != null) continue;
      
      var strClass = node.getAttribute("class");
      var bLazy = node.getAttribute("lazy") == "true";
      var bRequire = node.getAttribute("require") == "true";
      
      var objClass = jsx3.Class.forName(strClass);
      if (objClass == null && bRequire) {
        if (this._classesToLoad == null) {
          
          this._classesToLoad = [];
        }
        this._classesToLoad.push(strClass);
        this._handlers[strName] = Manager.LAZY_HANDLER;
        continue;
      }
      
      if (objClass) {
        var handler = objClass.newInstance(strName);
        this._initBean(handler, node);
        handler.onAfterInit();
        
        var strLevel = node.getAttribute('level');
        if (strLevel && typeof(Logger[strLevel]) == "number")
          handler.setLevel(Logger[strLevel]);
        
        this.addHandler(handler);
        initedNames[initedNames.length] = strName;
      } else if (!bLazy && !bRequire) {
        window.alert(jsx3._msg("logr.no_class", strClass));
      } else {
        this._handlers[strName] = Manager.LAZY_HANDLER;
      }
    }

    if (bNeedsJob && this._classesToLoad != null) {
      var me = this;
      var job = new jsx3.util.Job("logger.require");
      job.run = function() {
        // make this asynchronous to prevent race conditions from causing double-registration of required handlers
        jsx3.sleep(function() {this._onClassLoaderRest();}, null, me);
      };
      jsx3.CLASS_LOADER.addJob(job, "jsx.js");
    }
    
    return initedNames;
  };
  
  
  Manager_prototype._onClassLoaderRest = function(objEvent) {
    for (var i = 0; i < this._classesToLoad.length; i++)
      jsx3.require(this._classesToLoad[i]);
    
    delete this._classesToLoad;

    var loaded = this._initHandlers("[@require='true']");
    this._wireNewHandlers(loaded);
  };
  
  
  Manager_prototype._initBean = function(obj, objNode) {
    var objClass = obj.getClass();

    for (var i = objNode.selectNodeIterator("./property"); i.hasNext(); ) {
      var child = i.next();

      var strName = child.getAttribute("name");
      var strValue = child.getAttribute("value");
      var bEval = child.getAttribute("eval") == "true";

      var objSetter = objClass.getSetter(strName);
      if (objSetter != null) {
        if (bEval) {
          try {
            strValue = isNaN(strValue) ? obj.eval(strValue) : Number(strValue);
          } catch (e) {
            throw new Exception(jsx3._msg("logr.bn_eval", strName, strValue, obj), jsx3.NativeError.wrap(e));
          }
        }
        objSetter.apply(obj, [strValue]);
      } else {
        throw new Exception(jsx3._msg("logr.bn_setr", strName, objClass));
      }
    }
  };
  
    
  Manager_prototype.addLogger = function(objLogger) {
    var strName = objLogger.getName();
    this._loggers[strName] = objLogger;

    if (this._config) {
      var loggerNode = this._config.selectSingleNode("/configuration/logger[@name='" + strName + "']");

      // configure new logger from its own node
      if (loggerNode != null) {
        var strLevel = loggerNode.getAttribute('level');
        if (strLevel && typeof(Logger[strLevel]) == "number")
          objLogger.setLevel(Logger[strLevel]);

        var bUseParent = loggerNode.getAttribute('useParent') != "false";
        objLogger.setUseParentHandlers(bUseParent);

        var i = loggerNode.selectNodeIterator("./handler-ref");
        while (i.hasNext()) {
          var handlerNode = i.next();
          var strId = handlerNode.getAttribute('name');
          var handler = this.getHandler(strId);
          if (handler != null) {
            objLogger.addHandler(handler);
          } else if (this._handlers[strId] != Manager.LAZY_HANDLER) {
            throw new Exception(jsx3._msg("logr.no_hand", strName, strId));
          }
        }

        this._initBean(objLogger, loggerNode);
      }
    }
    
    // find its parent
    if (strName != Manager.ROOT_KEY) {
      var index = strName.lastIndexOf(".");
      var parentName = index >= 0 ? strName.substring(0, index) : Manager.ROOT_KEY;
      // recursive call to create the parent
      objLogger.setParent(Logger.getLogger(parentName));
    }
    
    // no need to find children among existing loggers, because parents are always initialized with children
  };
  
  
  Manager_prototype.addHandler = function(objHandler) {
    this._handlers[objHandler.getName()] = objHandler;
  };

  
  Manager_prototype.getLogger = function(strName) {
    return this._loggers[strName];
  };
  
  
  Manager_prototype.getHandler = function(strName) {
    var handler = this._handlers[strName];
    return handler == Manager.LAZY_HANDLER ? null : handler;
  };
  
  
  Manager_prototype.getHandlerNames = function() {
    var keys = [];
    for (var f in this._handlers)
      keys[keys.length] = f;
    return keys;
  };
  
});


jsx3.Class.defineClass('jsx3.util.Logger.Record', null, null, function(Record, Record_prototype) {

  
  Record.SERIAL = 1;
  
  
  Record_prototype._serial = null;
  
  Record_prototype._date = null;
  
  Record_prototype._message = null;
  
  Record_prototype._params = null;
  
  Record_prototype._level = null;
  
  Record_prototype._logger = null;
  
  Record_prototype._stack = null;
  
  Record_prototype._error = null;

  
  Record_prototype.init = function(strMessage, arrParams, intLevel, strLogger, arrStack, objError) {
    this._serial = Record.SERIAL++;
    this._date = new Date();
    this._message = strMessage;
    this._params = arrParams;
    this._level = intLevel;
    this._logger = strLogger;
    this._stack = arrStack;
    this._error = objError;
  };
  
  
  Record_prototype.getSerial = function() {
    return this._serial;
  };
  
  
  Record_prototype.getDate = function() {
    return this._date;
  };
  
  
  Record_prototype.getMessage = function() {
    return this._message;
  };
  
  
  Record_prototype.getParameters = function() {
    return this._params;
  };
  
  
  Record_prototype.getLevel = function() {
    return this._level;
  };
  
  
  Record_prototype.getLoggerName = function() {
    return this._logger;
  };
  
  
  Record_prototype.getFunction = function() {
    return typeof(this._stack) == "function" ? this._stack : null;
  };
  
  
  Record_prototype.getStack = function() {
    return this._stack instanceof Array ? this._stack : null;
  };
  
  
  Record_prototype.getError = function() {
    return this._error;
  };
  
});


jsx3.Class.defineClass('jsx3.util.Logger.Handler', null, null, function(Handler, Handler_prototype) {

  var Logger = jsx3.util.Logger;
  var Manager = jsx3.util.Logger.Manager;
  
  
  Handler.registerHandlerClass = function(objClass) {
    Manager.getManager()._handlerClassDidLoad(objClass);
  };
  
  
  Handler_prototype._name = "";
  
  Handler_prototype._level = null;
  
  
  Handler_prototype.init = function(strName) {
    this._name = strName;
  };

  Handler_prototype.onAfterInit = function() {};
  
  
  Handler_prototype.getName = function() {
    return this._name;
  };
  
  
  Handler_prototype.getLevel = function() {
    return this._level;
  };
  
  
  Handler_prototype.setLevel = function(intLevel) {
    intLevel = Math.max(Logger.OFF, Math.min(Logger.MIN_LEVEL, intLevel));
    this._level = intLevel;
  };
    
  
  Handler_prototype.isLoggable = function(intLevel) {
    return this._level == null || this._level >= intLevel;
  };
  
  
  Handler_prototype.handle = jsx3.Method.newAbstract('objRecord');
  
});


jsx3.Class.defineClass('jsx3.util.Logger.MemoryHandler', jsx3.util.Logger.Handler, null, function(MemoryHandler, MemoryHandler_prototype) {

  
  MemoryHandler_prototype._buffer = null;
  
  MemoryHandler_prototype._bufferSize = 100;

  
  MemoryHandler_prototype.init = function(strName) {
    this.jsxsuper(strName);
    this._buffer = [];
  };
  
  
  MemoryHandler_prototype.handle = function(objRecord) {
    // TODO: make more efficient, is Array implementation a linked list? probably not
    this._buffer[this._buffer.length] = objRecord;
    
    if (this._buffer.length > this._bufferSize)
      this._buffer.shift();
  };
  
  
  MemoryHandler_prototype.clearBuffer = function() {
    this._buffer = [];
  };
  
  
  MemoryHandler_prototype.getBufferSize = function() {
    return this._bufferSize;
  };
  
  
  MemoryHandler_prototype.setBufferSize = function(intBufferSize) {
    this._bufferSize = Math.max(1, intBufferSize);
    
    if (this._buffer.length > this._bufferSize)
      this._buffer.splice(0, this._buffer.length - this._bufferSize);
  };
  
  
  MemoryHandler_prototype.getRecords = function(intCount) {
    if (intCount == null) intCount = this._buffer.length;
    return this._buffer.slice(this._buffer.length - intCount);
  };
  
});


jsx3.Class.defineClass('jsx3.util.Logger.FormatHandler', jsx3.util.Logger.Handler, null, function(FormatHandler, FormatHandler_prototype) {

  
  FormatHandler_prototype._format = "%d %t %n (%l) - %M";
  
  FormatHandler_prototype._dynaPropUrls = "";

  FormatHandler_prototype.init = function(strName) {
    this.jsxsuper(strName);
  };
  
  FormatHandler_prototype.format = function(objRecord) {
    var date = objRecord.getDate();
    var funct = objRecord.getFunction();
    var message = objRecord.getMessage() || ""; // TODO: need to do message format

    var mf = this._getMessageFormat();
    var levelString = jsx3.util.Logger.levelAsString(objRecord.getLevel());
    // mf may be null if the MessageFormat class is not available
    var output = mf ? mf.format(
        objRecord.getSerial(),
        objRecord.getLoggerName(),
        levelString,
        message,
        funct != null ? (funct.jsxmethod != null ? funct.jsxmethod.toString() : funct.toString()) : "",
        date
      ) : date + " " + objRecord.getLoggerName() + " (" + levelString + ") " + message;
    var error = objRecord.getError();
    var stack = objRecord.getStack();
    
    if (error != null) {
      output += "\n" + error.printStackTrace();
    } else if (stack != null) {
      output += "\n" + jsx3.Exception.formatStack(stack);
    }
    
    return output;
  };
  
  
  FormatHandler_prototype.getFormat = function() {
    return this._format;
  };
  
  
  FormatHandler_prototype.setFormat = function(strFormat) {
    this._format = strFormat;
    this._messageformat = null;
  };
  
  
  FormatHandler_prototype._getMessageFormat = function() {
    // jsx3.util.MessageFormat class is optional, should work without it
    if (this._messageformat == null && jsx3.util.MessageFormat) {
      var f = this._format || "";
      f = f.replace(/\%s/g, "{0}");
      f = f.replace(/\%n/g, "{1}");
      f = f.replace(/\%l/g, "{2}");
      f = f.replace(/\%M/g, "{3}");
      f = f.replace(/\%f/g, "{4}");
      f = f.replace(/\%d/g, "{5,date,yyyy-MM-dd}");
      f = f.replace(/\%t/g, "{5,date,HH:mm:ss.SSS}");
      
      this._messageformat = new jsx3.util.MessageFormat(f);
    }
    return this._messageformat;
  };

  FormatHandler_prototype.getResourceUrls = function() {
    return this._dynaPropUrls;
  };
  
  
  FormatHandler_prototype.setResourceUrls = function(strURLs) {
    this._dynaPropUrls = strURLs;
  };
  
});


jsx3.Class.defineClass('jsx3.util.Logger.AlertHandler', jsx3.util.Logger.FormatHandler, null,
    function(AlertHandler, AlertHandler_prototype) {

  
  AlertHandler_prototype._interval = 10;
  
  AlertHandler_prototype._counter = 0;
  
  AlertHandler_prototype._disabled = false;

  AlertHandler_prototype.handle = function(objRecord) {
    if (this._disabled) return;
    this._counter++;

    try {
      if (this._interval > 0 && (this._counter % this._interval) == 0) {
        if (! window.confirm(jsx3._msg("logr.alrt_ctd", this.getName()))) {
          this._disabled = true;
          return;
        }
      }

      window.alert(this.format(objRecord));
    } catch (e) {
      window.alert(jsx3._msg("logr.alrt_err", jsx3.NativeError.wrap(e)));
    }
  };

  
  AlertHandler_prototype.getConfirmInterval = function() {
    return this._interval;
  };

  
  AlertHandler_prototype.setConfirmInterval = function(intInterval) {
    this._interval = intInterval;
  };

});




jsx3.ERROR = jsx3.util.Logger;


