


jsx3.Package.definePackage("jsx3.util", function(util) {

  
  util.compareVersions = function(v1, v2) {
    var regex = /^(\d+)?([a-zA-Z_]\w*)?$/;

    var v1t = v1.split(/\./);
    var v2t = v2.split(/\./);
    var maxLength = Math.max(v1t.length, v2t.length);

    var ad, al, bd, bl;

    for (var i = 0; i < maxLength; i++) {
      if (v1t.length > i && regex.test(v1t[i])) {
        ad = parseInt(RegExp.$1) || Number(0);
        al = RegExp.$2;
      } else {
        ad = 0;
        al = "";
      }

      if (v2t.length > i && regex.test(v2t[i])) {
        bd = parseInt(RegExp.$1) || Number(0);
        bl = RegExp.$2;
      } else {
        bd = 0;
        bl = "";
      }

      if (ad > bd) return 1;
      if (ad < bd) return -1;
      if (al > bl) return 1;
      if (al < bl) return -1;
    }

    return 0;
  };

  
  util.numMod = function(v, mod) {
    var result = v % mod;
    return result < 0 ? result + mod : result;
  };

  
  util.numIsNaN = function(v) {
    return v == null || v === "" || isNaN(v);
  };

  
  util.numRound = function(v, intUnit) {
    return Math.round(v/intUnit) * intUnit;
  };

  
  util.strEmpty = function(s) {
    return s == null || s === "";
  };

  
  util._jsxescapemap = {};
  util._jsxescapemap['\b'] = '\\b';
  util._jsxescapemap['\t'] = '\\t';
  util._jsxescapemap['\n'] = '\\n';
  util._jsxescapemap['\f'] = '\\f';
  util._jsxescapemap['\r'] = '\\r';
  util._jsxescapemap['"'] = '\\"';
  util._jsxescapemap['\\'] = '\\\\';

  
  util.strEscapeJSON = function (str) {
    if (/["\\\x00-\x1f]/.test(str)) {
      return '"' + str.replace(/[\x00-\x1f\\"]/g, function (a) {
        var c = util._jsxescapemap[a];
        if (c)
          return c;
        c = a.charCodeAt();
        return '\\u00' + Math.floor(c / 16).toString(16) + (c % 16).toString(16);
      }) + '"';
    }
    return '"' + str + '"';
  };

  
  util.arrIndexOf = function(a, o) {
    for (var i = 0; i < a.length; i++)
      if (a[i] === o) return i;
    return -1;
  };

  
  util._TRIM_REGEX = /(^\s*)|(\s*$)/g;

  
  util.strTrim = function(s) {
    return s.replace(util._TRIM_REGEX, "");
  };

  
  util.strEscapeHTML = function(s) {
    return String(s).replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/>/g,"&gt;").replace(/"/g,"&quot;").replace(
        // NOTE: the XML spec would be [^\x09\x0A\x0D\x20-\xD7FF\xE000-\xFFFD\x10000-\x10FFFF] but Firefox won't
        // compile that regex, see http://www.w3.org/TR/REC-xml/
        /[^\x09\x0A\x0D\x20-\x7F]/g,
        function(m) {
          var c = m.charCodeAt(0);
          if (c < 0x20 || (c > 0xD7FF && c < 0xE000) || (c > 0xFFFD && c < 0x10000) || c > 0x10FFFF)
            return "\\u" + c.toString(16);
          else
            return m;
        } );
  };

  
  util.strTruncate = function(s, intMax, strEllipsis, fltPos) {
    if (strEllipsis == null) strEllipsis = "...";
    if (fltPos == null) fltPos = 1.0;

    if (s.length > intMax && strEllipsis.length < intMax) {
      var l = intMax - strEllipsis.length;
      var beforeLength = Math.round(l * fltPos);
      var before = s.substring(0, beforeLength);
      var after = s.substring(s.length - (l - beforeLength));
      return before + strEllipsis + after;
    } else {
      return s;
    }
  };

  
  util.strEndsWith = function(s, strTest) {
    var index = s.lastIndexOf(strTest);
    return index >= 0 && index == s.length - strTest.length;
  };

  
  util._BASE64S = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

  
  util.strEncodeBase64 = function(s) {
    var base64s = util._BASE64S;
    var buffer = new Array(Math.ceil(s.length * 4 / 3));

    var i = 0, c = 0, length = s.length;
    for (; i <= length - 3; i += 3) {
      var bits = (s.charCodeAt(i)   & 0xff) << 16 |
                 (s.charCodeAt(i+1) & 0xff) << 8  |
                 (s.charCodeAt(i+2) & 0xff);

      buffer[c++] = base64s.charAt((bits & 0xfc0000) >> 18);
      buffer[c++] = base64s.charAt((bits & 0x03f000) >> 12);
      buffer[c++] = base64s.charAt((bits & 0x000fc0) >> 6);
      buffer[c++] = base64s.charAt((bits & 0x00003f));
    }

    if (i < length) {
      var dual = i < length - 1;

      var bits = (s.charCodeAt(i) & 0xff) << 16;
      if (dual)
        bits |= (s.charCodeAt(i+1) & 0xff) << 8;

      buffer[c++] =   base64s.charAt((bits & 0xfc0000) >> 18);
      buffer[c++] =   base64s.charAt((bits & 0x03f000) >> 12);
      if (dual)
        buffer[c++] = base64s.charAt((bits & 0x000fc0) >> 6);
      else
        buffer[c++] = "=";
      buffer[c++] = "=";
    }

    return buffer.join("");
  };

  
  util.strDecodeBase64 = function(s) {
    var base64s = util._BASE64S;
    var buffer = new Array(Math.ceil(s.length / 4));

    //declare variables
    var i = 0, c = 0, length = s.length;
    for (; i < length; i += 4) {
      var bits = (base64s.indexOf(s.charAt(i))   & 0xff) << 18 |
                 (base64s.indexOf(s.charAt(i+1)) & 0xff) << 12 |
                 (base64s.indexOf(s.charAt(i+2)) & 0xff) <<  6 |
                 (base64s.indexOf(s.charAt(i+3)) & 0xff);

      buffer[c++] = String.fromCharCode((bits & 0xff0000) >> 16, (bits & 0xff00) >> 8, bits & 0xff);
    }

    if (s.charCodeAt(i-2) == 61)
      buffer[c-1] = buffer[c-1].substring(0,1);
    else if (s.charCodeAt(i-1) == 61)
      buffer[c-1] = buffer[c-1].substring(0,2);

    return buffer.join("");
  };

});




jsx3.Class.defineClass("jsx3.util.Timer", null, null, function(Timer, Timer_prototype) {
  
  
  Timer._LISTENERS = [];
  
  
  Timer.listen = function() {
    var l = [];
    Timer._LISTENERS.push(l);
    return l;
  };

  
  Timer.ignore = function(arrStats) {
    var i = jsx3.util.arrIndexOf(Timer._LISTENERS, arrStats);
    Timer._LISTENERS.splice(i, 1);
  };
  
  
  Timer._record = function(objTimer, strMessage, intMS) {
    var l = Timer._LISTENERS;
    if (l.length > 0) {
      var o = {topic:objTimer._topic, subtopic:objTimer._message, message:strMessage, ms:intMS};
      for (var i = 0; i < l.length; i++) {
        l[i].push(o);
      }
    }
  };

  
  Timer_prototype.init = function(strTopic, strSubtopic, intLevel) {
    
    this._t1 = new Date();
    
    this._topic = String(strTopic);
    
    this._message = String(strSubtopic);
    
    this._level = intLevel || 4; // default is INFO
  };

  
  Timer_prototype.log = function(strMessage) {
    var t2 = new Date();
    if (! this._logger) {
      if (jsx3.util.Logger)
        
        this._logger = jsx3.util.Logger.getLogger("bench." + this._topic);
    }

    var timeTaken = t2 - this._t1;
    if (this._logger && this._logger.isLoggable(this._level))
      this._logger.log(this._level, 
          (this._message ? this._message  + " : " : "") + (strMessage ? strMessage + " : ": "") + timeTaken + "ms");

    Timer._record(this, strMessage, timeTaken);
    this._t1 = t2;
  };
  
});


jsx3.Class.defineClass("jsx3.util.WeakMap", null, null, function(WeakMap, WeakMap_prototype) {

  
  WeakMap._SERIAL = 0;
  
  WeakMap._ALL = {};
  
  WeakMap._TO = null;

  
  WeakMap.expire = function() {
    window.clearTimeout(WeakMap._TO);
    WeakMap._TO = null;
    for (var f in WeakMap._ALL) {
      var m = WeakMap._ALL[f];
      if (m._dirty) m._map = {};
    }
  };

  
  WeakMap_prototype.init = function() {
    
    this._serial = WeakMap._SERIAL++;
    
    this._map = {};
    
    this._dirty = false;

    WeakMap._ALL[this._serial] = this;
  };

  
  WeakMap_prototype.get = function(strKey) {
    return this._map[strKey];
  };

  
  WeakMap_prototype.set = function(strKey, strValue) {
    this._map[strKey] = strValue;
    this._dirty = true;
    if (! WeakMap._TO) 
      WeakMap._TO = window.setTimeout(WeakMap.expire, 0);
  };

  
  WeakMap_prototype.destroy = function(strKey, strValue) {
    delete this._map;
    delete WeakMap._ALL[this._serial];
  };

});


